Package: natserv
Title: 'NatureServe' Interface
Description: Interface to 'NatureServe' (<https://www.natureserve.org/>).
    Includes methods to get data, image metadata, search taxonomic names,
    and make maps.
Version: 1.0.0
License: MIT + file LICENSE
URL: https://docs.ropensci.org/natserv,
        https://github.com/ropensci/natserv
BugReports: https://github.com/ropensci/natserv/issues
Authors@R: c(
    person("Scott", "Chamberlain", role = c("aut", "cre"),
        email = "myrmecocystus@gmail.com",
        comment = c(ORCID="0000-0003-1444-9135")),
    person(given = "rOpenSci",
        role = "fnd",
        comment = "https://ropensci.org/"))
LazyData: true
VignetteBuilder: knitr
Language: en-US
Encoding: UTF-8
Depends: R(>= 3.2.1)
Imports: crul (>= 0.7.0), jsonlite, tibble
Suggests: testthat, knitr, vcr
RoxygenNote: 7.1.0
X-schema.org-applicationCategory: Taxonomy
X-schema.org-keywords: taxonomy, species, API, web-services,
        NatureServe, metadata, maps
X-schema.org-isPartOf: https://ropensci.org
NeedsCompilation: no
Packaged: 2020-05-15 21:02:58 UTC; sckott
Author: Scott Chamberlain [aut, cre] (<https://orcid.org/0000-0003-1444-9135>),
  rOpenSci [fnd] (https://ropensci.org/)
Maintainer: Scott Chamberlain <myrmecocystus@gmail.com>
Repository: CRAN
Date/Publication: 2020-05-16 22:30:07 UTC
Built: R 4.6.0; ; 2026-04-26 08:43:16 UTC; unix

# Generated by roxygen2: do not edit by hand

export(ns_altid)
export(ns_ecohier)
export(ns_export)
export(ns_export_status)
export(ns_id)
export(ns_search_comb)
export(ns_search_eco)
export(ns_search_spp)
importFrom(crul,HttpClient)
importFrom(jsonlite,fromJSON)
importFrom(tibble,as_tibble)
importFrom(tibble,enframe)

natserv
=======



[![cran checks](https://cranchecks.info/badges/worst/natserv)](https://cranchecks.info/pkgs/natserv)
[![Build Status](https://travis-ci.org/ropensci/natserv.svg?branch=master)](https://travis-ci.org/ropensci/natserv)
[![Build status](https://ci.appveyor.com/api/projects/status/mvmi4h4jn5ixf3hs?svg=true)](https://ci.appveyor.com/project/sckott/natserv)
[![codecov](https://codecov.io/gh/ropensci/natserv/branch/master/graph/badge.svg)](https://codecov.io/gh/ropensci/natserv)
[![cran version](https://www.r-pkg.org/badges/version/natserv)](https://cran.r-project.org/package=natserv)
[![rstudio mirror downloads](https://cranlogs.r-pkg.org/badges/natserv)](https://github.com/metacran/cranlogs.app)


`natserv` NatureServe R client

NatureServe is a non-profit organization that provides wildlife conservation related data to various groups including the public.

* NatureServe site: https://services.natureserve.org
* NatureServe API docs: https://explorer.natureserve.org/api-docs/

All functions in this package are prefixed with `ns_` to prevent
collision with other pkgs.

You no longer need an API key.

See also the taxize book (https://taxize.dev/) for 
a manual on working with taxonomic data in R, including with NatureServe data.

## Installation

Stable version from CRAN


```r
install.packages("natserv")
```

Development version


```r
remotes::install_github("ropensci/natserv")
```


```r
library('natserv')
```

## Search


```r
ns_search_comb(text = "robin", page = 0, per_page = 5)
#> $results
#> # A tibble: 5 x 14
#>   recordType elementGlobalId uniqueId nsxUrl elcode scientificName
#>   <chr>                <int> <chr>    <chr>  <chr>  <chr>         
#> 1 SPECIES             100637 ELEMENT… /Taxo… ABPBJ… Copsychus sau…
#> 2 SPECIES             102323 ELEMENT… /Taxo… ABPBJ… Turdus grayi  
#> 3 SPECIES             102179 ELEMENT… /Taxo… ABPBJ… Turdus migrat…
#> 4 SPECIES             105536 ELEMENT… /Taxo… ABPBJ… Turdus migrat…
#> 5 SPECIES             105850 ELEMENT… /Taxo… ABPBJ… Turdus rufopa…
#> # … with 22 more variables: formattedScientificName <chr>,
#> #   primaryCommonName <chr>, primaryCommonNameLanguage <chr>,
#> #   roundedGRank <chr>, nations <list>, lastModified <chr>,
#> #   speciesGlobal$usesaCode <lgl>, $cosewicCode <lgl>, $saraCode <lgl>,
#> #   $synonyms <list>, $otherCommonNames <list>, $kingdom <chr>, $phylum <chr>,
#> #   $taxclass <chr>, $taxorder <chr>, $family <chr>, $genus <chr>,
#> #   $taxonomicComments <chr>, $informalTaxonomy <chr>, $infraspecies <lgl>,
#> #   $completeDistribution <lgl>, gRank <chr>
#> 
#> $resultsSummary
#>                            name value
#> 1                          page     0
#> 2                recordsPerPage     5
#> 3                    totalPages    26
#> 4                  totalResults   126
#> 5                 species_total   103
#> 6                         total    23
#> 7                       classes     0
#> 8                    subclasses     0
#> 9                    formations     0
#> 10                    divisions     0
#> 11                  macrogroups     1
#> 12                       groups     1
#> 13                    alliances     3
#> 14                 associations    18
#> 15 terrestrialEcologicalSystems     0
```

See the vignette (https://docs.ropensci.org/natserv/articles/natserv.html) for more examples.

## Meta

* Please [report any issues or bugs](https://github.com/ropensci/natserv/issues).
* License: MIT
* Get citation information for `natserv` in R doing `citation(package = 'natserv')`
* Please note that this project is released with a [Contributor Code of Conduct][coc]. By participating in this project you agree to abide by its terms.

[![ropensci](https://ropensci.org/public_images/github_footer.png)](https://ropensci.org)

[coc]: https://github.com/ropensci/natserv/blob/master/CODE_OF_CONDUCT.md

mssg <- function(v, ...) if (v) message(...)
tc <- function(l) Filter(Negate(is.null), l)
tcnull <- function(x) if (all(sapply(x, is.null))) NULL else x

pluck <- function(x, name, type) {
  if (missing(type)) {
    lapply(x, "[[", name)
  } else {
    vapply(x, "[[", name, FUN.VALUE = type)
  }
}

`%||%` <- function(x, y) if (is.null(x) || length(x) == 0) y else x
`%|lst|%` <- function(x, y) if (length(unlist(x)) == 0) y else x

strtrim <- function(str) {
  gsub("^\\s+|\\s+$", "", str)
}

str_extrct <- function(string, pattern) {
  regmatches(string, regexpr(pattern, string))
}

assert <- function(x, y) {
  if (!is.null(x)) {
    if (!class(x) %in% y) {
      stop(deparse(substitute(x)), " must be of class ",
           paste0(y, collapse = ", "), call. = FALSE)
    }
  }
}

check_uid <- function(x) {
  if (!is.null(x)) {
    greps <- grepl("ELEMENT_GLOBAL", x)
    if (any(!greps)) {
      stop("one or more 'uid's appear not to be NatureServe ID's:\n",
        paste0(x[!greps], collapse=", "),
        call. = FALSE)
    }
  }
}

context("internal tooling")

test_that("check_uid", {
  # bad
  expect_error(check_uid("adsfadf"), 'one or more')
  expect_error(check_uid("adsfadf"), 'adsfadf')

  expect_error(check_uid(c("foo", "bar")), 'foo, bar')

  # good
  expect_null(check_uid("ELEMENT_GLOBAL.2.104470"))
  
  # if null, passes
  expect_null(check_uid(NULL))
})

test_that("internal tooling", {
  # tc
  expect_error(tc())
  expect_equal(tc(5), 5)
  expect_equal(tc(list(5, NULL)), list(5))

  # pluck
  x <- list(list(foo = 5, stuff = 4), list(foo = 5), list(foo = 7, bar = 5))
  expect_error(pluck())
  expect_equal(pluck(x, "foo"), list(5, 5, 7))
  expect_equal(pluck(x, "foo", 1), c(5, 5, 7))

  # %||%
  expect_equal(5 %||% 6, 5)
  expect_equal(NULL %||% 5, 5)

  # %|lst|%
  expect_equal(list() %|lst|% 6, 6)
  expect_equal(list(list()) %|lst|% 6, 6)

  # strtrim
  expect_equal(strtrim(" adf "), "adf")

  # str_extrct
  expect_equal(str_extrct("stuff -things", "-.+"), "-things")

  # assert
  z <- 5L
  expect_error(assert(z, "character"))
  expect_null(assert(z, "integer"))
})

#' Map NatureServe data
#'
#' @export
#' @name ns_map
#' @param x the result of a call to \code{\link{ns_data}}
#' @param ... ignored
#' @examples \dontrun{
#' # Aquila chrysaetos
#' x <- ns_data(uid = 'ELEMENT_GLOBAL.2.100925')
#' ns_map_county(x)
#' ns_map_cons(x)
#' ns_map_stpr(x)
#'
#' # Ursus americanus
#' x <- ns_data('ELEMENT_GLOBAL.2.100661')
#' ns_map_county(x)
#' ns_map_cons(x)
#' ns_map_stpr(x)
#' }
ns_map_county <- function(x, ...) {
  chek_pk("mapproj")
  chek_pk("ggplot2")
  chek_pk("maps")
  stopifnot(inherits(x, "list"))
  co <- x[[1]]$distribution$countyDistribution
  if (length(co) == 0) stop("no county distribution data", call. = FALSE)
  co <- co[[1]]
  co <- Map(function(x, y) {
    x$state <- y
    x
  }, co, names(co))
  dat <- data.table::setDF(
    data.table::rbindlist(co, fill = TRUE, use.names = TRUE)
  )
  dat$countyName <- tolower(dat$countyName)
  dat$maximumLastObservedYear <- as.numeric(dat$maximumLastObservedYear)
  dat$speciesOccurrenceCount <- as.numeric(dat$speciesOccurrenceCount)

  dat <- merge(dat, nat_states, by = "state")

  counties <- ggplot2::map_data("county")
  counties_plus <- merge(
    counties,
    dat,
    by.x = c('region', 'subregion'),
    by.y = c('state_name', 'countyName'),
    all = TRUE)
  counties_plus <- counties_plus[order(counties_plus$order),]
  states <- ggplot2::map_data("state")
  ggplot2::ggplot(counties_plus, ggplot2::aes(long, lat, group = group)) +
    ggplot2::geom_polygon(ggplot2::aes(fill = speciesOccurrenceCount)) +
    ggplot2::coord_map(projection = "azequalarea") +
    ggplot2::scale_fill_gradient2("", na.value = "lightgrey", low = "white", high = "steelblue") +
    ggplot2::geom_path(data = counties, colour = "grey", size = .3, alpha = .4) +
    ggplot2::geom_path(data = states, colour = "grey", size = .4) +
    ggplot2::theme_bw(base_size = 14) +
    ggplot2::labs(x = "", y = "") +
    map_blanktheme() +
    ggplot2::scale_x_continuous(expand = c(0,0)) +
    ggplot2::scale_y_continuous(expand = c(0,0))
}

#' @export
#' @rdname ns_map
ns_map_cons <- function(x, ...) {
  chek_pk("mapproj")
  chek_pk("ggplot2")
  chek_pk("maps")
  nst <- x[[1]]$conservationStatus$natureserve$nationalStatuses
  nst <- Filter(function(x) NROW(x) > 0, Map(function(a, b) {
    tmp <- lapply(a$subnationalStatuses, function(x) {
      utils::modifyList(x, list(rank = status_pick(strsplit(x$rank, ",")[[1]][[1]]) ))
    })
    df <- data.table::setDF(
      data.table::rbindlist(tmp, fill = TRUE, use.names = TRUE)
    )
    if (NROW(df) > 0 ) df$country <- b
    df
  }, nst, names(nst)))
  nst <- do.call("rbind", unname(nst))
  nst$subnationName <- tolower(nst$subnationName)
  states <- ggplot2::map_data("state")
  st_plus <- merge(states, nst, by.x = 'region', by.y = 'subnationName', all.x = TRUE)
  st_plus <- st_plus[order(st_plus$order), ]
  ggplot2::ggplot(st_plus, ggplot2::aes(long, lat, group = group)) +
    ggplot2::geom_polygon(ggplot2::aes(fill = rank)) +
    ggplot2::coord_map(projection = "azequalarea") +
    ggplot2::geom_path(data = st_plus, colour = "grey", size = .4) +
    ggplot2::theme_bw(base_size = 14) +
    ggplot2::labs(x = "", y = "") +
    map_blanktheme() +
    ggplot2::scale_x_continuous(expand = c(0,0)) +
    ggplot2::scale_y_continuous(expand = c(0,0))
}

#' @export
#' @rdname ns_map
ns_map_stpr <- function(x, ...) {
  chek_pk("mapproj")
  chek_pk("ggplot2")
  chek_pk("maps")
  na <- x[[1]]$distribution$nations
  na <- Filter(function(x) NROW(x) > 0, Map(function(a, b) {
    df <- data.table::setDF(
      data.table::rbindlist(a$subnations, fill = TRUE, use.names = TRUE)
    )
    if (NROW(df) > 0 ) df$country <- b
    df
  }, na, names(na)))
  na <- do.call("rbind", unname(na))
  na$subnationName <- tolower(na$subnationName)
  na$currentPresenceAbsence <- gsub("Present", "1", na$currentPresenceAbsence)
  na$currentPresenceAbsence <- gsub("Absent", "0", na$currentPresenceAbsence)
  na$currentPresenceAbsence <- as.numeric(na$currentPresenceAbsence)
  states <- ggplot2::map_data("state")
  st_plus <- merge(states, na, by.x = 'region', by.y = 'subnationName', all.x = TRUE)
  st_plus <- st_plus[order(st_plus$order),]

  ggplot2::ggplot(st_plus, ggplot2::aes(long, lat, group = group)) +
    ggplot2::geom_polygon(ggplot2::aes(fill = currentPresenceAbsence)) +
    ggplot2::coord_map(projection = "azequalarea") +
    ggplot2::geom_path(data = states, colour = "grey", size = .4) +
    ggplot2::theme_bw(base_size = 14) +
    ggplot2::labs(x = "", y = "") +
    map_blanktheme() +
    ggplot2::scale_x_continuous(expand = c(0,0)) +
    ggplot2::scale_y_continuous(expand = c(0,0))
}

status_pick <- function(w) {
  stats <- c("SX","SH","S1","S2","S3","S4","S5","SNR","SU","SNA",
             "SXB", "SHB", "S1B", "S2B", "S3B", "S4B",
             "S5B", "SNRB", "SUB", "SNAB", "SXN", "SHN",
             "S1N", "S2N", "S3N", "S4N", "S5N", "SNRN",
             "SUN", "SNAN", "SXM", "SHM", "S1M", "S2M",
             "S3M", "S4M", "S5M", "SNRM", "SUM", "SNAM")
  mm <- Filter(
    function(x) length(x) > 0,
    sapply(stats, function(z) str_extrct(w, z)))
  tmp <- if (length(mm) == 0) {
    NA_character_
  } else {
    mm[[1]]
  }
  if (tmp %in% c('SNR', 'SU', 'SNA')) NA else tmp
}

context("ns_map")

test_that("ns_map_county works as expected", {
  skip_on_cran()

  #dat <- ns_data(uid = 'ELEMENT_GLOBAL.2.100925')
  aa <- ns_map_county(ns_data_output)

  expect_is(aa, 'gg')
  expect_is(aa, 'ggplot')
  expect_is(aa$data, 'data.frame')
  expect_is(aa$layers, 'list')
})

test_that("ns_map_county works as expected", {
  skip_on_cran()

  #dat <- ns_data(uid = 'ELEMENT_GLOBAL.2.100925')
  aa <- ns_map_cons(ns_data_output)

  expect_is(aa, 'gg')
  expect_is(aa, 'ggplot')
  expect_is(aa$data, 'data.frame')
  expect_is(aa$layers, 'list')
})

test_that("ns_map_county works as expected", {
  skip_on_cran()

  #dat <- ns_data(uid = 'ELEMENT_GLOBAL.2.100925')
  aa <- ns_map_stpr(ns_data_output)

  expect_is(aa, 'gg')
  expect_is(aa, 'ggplot')
  expect_is(aa$data, 'data.frame')
  expect_is(aa$layers, 'list')
})

test_that("ns_map fails well", {
  skip_on_cran()

  expect_error(ns_map_county(),
               'argument "x" is missing')

  expect_error(ns_map_county(5),
               'inherits\\(x, "list"\\) is not TRUE')
})

#' NatureServe image metadata
#'
#' @export
#' @template ns
#' @param uid (character) a species UID, e.g., ELEMENT_GLOBAL.2.100925
#' @param scientificName (character) An asterisk (*) wildcarded species
#' scientific name, e.g., 'Aquila chry*'. Name matching is case-insensitive.
#' @param commonName (character) An asterisk (*) wildcarded species common
#' name, e.g., 'g*EAGLE'. Name matching is case-insensitive.
#' @param includeSynonyms (character)	An optional parameter, relevant to
#' scientific or common name queries, that indicates whether to include
#' synonymous names in the query, as follows:
#' \itemize{
#'  \item Y (or y) - search the Primary and all synonymous Scientific and
#'  Common Names
#'  \item Any other value, or omitted - search only the Primary Scientific
#'  and Common Name fields
#' }
#' @param resolution (character)	An optional parameter that restricts output
#' to images at a certain resolution. The value can be of one of the following.
#' \itemize{
#'  \item lowest - return only the lowest resolution version of images
#'  \item highest - return only the highest resolution version of images
#'  \item thumbnail - return only the version NatureServe deems the
#'  'thumbnail image'
#'  \item web - return only the version NatureServe deems the 'web image'
#' }
#' The omission of this parameter leads to the return of metadata for images at
#' all available resolutions.
#' @param ITISNames (character)	An optional parameter, relevant to scientific
#' or common name queries, that will indicate whether to restrict queries to
#' ITIS names, as follows.
#' \itemize{
#'  \item Y (or y) - query using ITIS names
#'  \item Any other value, or omitted - query using NatureServe names
#' }
#' NOTE: This parameter is a placeholder only and will not affect processing.
#' At present, searching by ITIS names is not possible.
#'
#' @details Note that the NatureServer servers apparently want Windows
#' HTML encoding (Windows-1252), instead of UTF-8, so some accents
#' and such may not work
#'
#' @return a list with terms and images
#'
#' @examples \dontrun{
#' # search by uid
#' ns_images(uid = 'ELEMENT_GLOBAL.2.100925')
#'
#' # search by common name and resolutio thumbnail
#' (res <- ns_images(commonName = "*eagle", resolution = 'thumbnail'))
#'
#' # search "Ruby*", all common names [in any language], and highest
#' # resolution only:
#' (res <- ns_images(commonName = "Ruby*", includeSynonyms = 'y',
#'   resolution = 'highest'))
#' }
ns_images <- function(uid = NULL, scientificName = NULL, commonName = NULL,
                      includeSynonyms = NULL, resolution = NULL,
                      ITISNames = NULL, key = NULL, ...) {

  assert(uid, "character")
  check_uid(uid)
  assert(scientificName, "character")
  assert(commonName, "character")
  assert(includeSynonyms, "character")
  assert(resolution, "character")
  assert(ITISNames, "character")
  args <- tc(list(uid = uid, scientificName = scientificName,
                  commonName = commonName, includeSynonyms = includeSynonyms,
                  resolution = resolution,
                  ITISNames = ITISNames, NSAccessKeyId = check_key(key)))
  res <- ns_GET(
    url = paste0(ns_base(), '/v1/globalSpecies/images'),
    query = args,
    err_fxn = err_catch_images,
    ...
  )
  xml <- xml2::read_xml(res)
  list(
    terms = strtrim(xml2::xml_text(
      xml2::xml_find_first(xml, "d1:termsAndConditions"))),
    images = lapply(xml2::xml_find_all(xml, "d1:image"), function(w) {
      d1 <- xml2::xml_find_all(w, "d1:*")
      d1 <- d1[which(xml2::xml_name(d1) != "imagePermission")]
      c(
        sapply(d1, function(y) as.list(stats::setNames(xml2::xml_text(y),
                                                       xml2::xml_name(y)))),
        list(
          imagePermission =
            sapply(
              xml2::xml_children((xml2::xml_find_all(w, "d1:imagePermission"))),
              function(y) as.list(stats::setNames(xml2::xml_text(y),
                                                  xml2::xml_name(y))))),
        sapply(xml2::xml_find_all(w, "dc:*"), function(y)
          as.list(stats::setNames(xml2::xml_text(y), xml2::xml_name(y))))
      )
    })
  )
}

context("ns_images")

test_that("ns_images works as expected", {
  skip_on_cran()

  vcr::use_cassette("ns_images", {
    aa <- ns_images(uid = 'ELEMENT_GLOBAL.2.100925')
  })

  expect_is(aa, 'list')
  expect_named(aa, c('terms', 'images'))
  expect_is(aa$terms, 'character')
  expect_is(aa$images, 'list')
  expect_is(aa$images[[1]], 'list')
  expect_is(aa$images[[1]]$id, 'character')
})

test_that("ns_images fails well", {
  skip_on_cran()

  vcr::use_cassette("ns_images_error", {
    expect_error(ns_images(scientificName = "asdfasf"),
                 'no results found')
    expect_error(ns_images(commonName = "asdfasf"),
                 'no results found')
  })

  expect_error(ns_images("adfdf"),
               "one or more 'uid's")

  # fails well when input not character
  expect_error(ns_images(5), 'uid must be of class character')
  expect_error(ns_images(TRUE), 'uid must be of class character')

  expect_error(ns_images(scientificName = 5), 'scientificName must be of class character')
  expect_error(ns_images(commonName = 5), 'commonName must be of class character')
  expect_error(ns_images(includeSynonyms = 5), 'includeSynonyms must be of class character')
  expect_error(ns_images(resolution = 5), 'resolution must be of class character')
  expect_error(ns_images(ITISNames = 5), 'ITISNames must be of class character')
})

#' Ecosystem search
#'
#' @export
#' @inheritParams ns_search_comb
#' @param ecosystem_taxonomy (character) the classification code of the
#' higher level (ancestor) ecosystem. E.g.'s: "1" (Class code),
#' "1.B" (Subclass code), "1.B.2" (Formation code), "1.B.2.Nd" (Division code),
#' "M886" (Macrogroup key), "G206" (Group key), "A3328" (Alliance Key)
#' @template ns
#' @family search
#' @examples \dontrun{
#' ns_search_eco(text = "robin")
#' ns_search_eco(text_adv = list(searchToken = "bird",
#'   matchAgainst = "allNames", operator="similarTo"))
#' ns_search_eco(status = "G1")
#' ns_search_eco(location = list(nation = "US"))
#' ns_search_eco(location = list(nation = "US", subnation = "VA"))
#' ns_search_eco(ecosystem_taxonomy = "M067")
#' ns_search_eco(record_subtype = "macrogroup")
#' ns_search_eco(modified_since = "2020-04-30T00:00:00+0000")
#' ns_search_eco(page = 0, per_page = 2)
#' }
ns_search_eco <- function(text = NULL, text_adv = NULL, status = NULL,
  location = NULL, ecosystem_taxonomy = NULL, record_subtype = NULL,
  modified_since = NULL, page = NULL, per_page = NULL, ...) {

  paging <- handle_paging(page, per_page)
  text <- handle_text(text, text_adv)
  status <- handle_status(status)
  location <- handle_location(location)
  ecosystem_taxonomy <- handle_ecotax(ecosystem_taxonomy)
  record_subtype <- handle_subtype(record_subtype)
  assert(modified_since, "character")
  res <- ns_POST(
    url = file.path(ns_base(), 'api/data/ecosystemsSearch'),
    body = list(criteriaType = "ecosystems",
      textCriteria = list(text) %|lst|% NULL,
      statusCriteria = list(status) %|lst|% NULL,
      locationCriteria = list(location) %|lst|% NULL,
      ecosystemsTaxonomyCriteria = list(ecosystem_taxonomy) %|lst|% NULL,
      recordSubtypeCriteria = list(record_subtype) %|lst|% NULL,
      pagingOptions = paging,
      modifiedSince = modified_since
    ),
    ...
  )
  parse_search(res)
}

#' Combined search
#'
#' @export
#' @param text (character) basic text search, equiavalent to `text_adv`
#' with `matchAgainst="allNames"` and `operator="similarTo"`
#' @param text_adv (list) advanced search, must specify the following three
#' elements: `searchToken`, `matchAgainst`, and `operator`. see 
#' https://explorer.natureserve.org/api-docs/#_advanced_text_search_parameter
#' @param status (character) conservation status, one of G1, G2, G3, G4,
#' G5, GH, GX, GNR, GNA, GU. case insensitive
#' @param location (list) location, country and sub-country. specify either
#' `nation` OR `nation` and `subnation`. each expects a two-letter ISO code
#' @param record_type (character) limit results by record type, one of
#' "species" or "ecosystem"
#' @param record_subtype (character) limit results by record sub-type, one of:
#' "class", "subclass", "formation", "division", "macrogroup", "group",
#' "alliance", "association", "terrestrial_ecological_system"
#' @param modified_since (character) search for records modified since a
#' given time. value must be a date and time with a UTC offset in ISO 8601
#' format. optional
#' @param page (integer) Zero-indexed page number; default: 0. optional
#' @param per_page (integer) Records per page; default: 20. optional
#' @template ns
#' @family search
#' @examples \dontrun{
#' ns_search_comb(text = "robin")
#' ns_search_comb(text_adv = list(searchToken = "western",
#'   matchAgainst="allScientificNames", operator="startsWith"))
#' ns_search_comb(status = "G1")
#' ns_search_comb(location = list(nation = "US"))
#' ns_search_comb(location = list(nation = "US", subnation = "VA"))
#' ns_search_comb(record_type = "species")
#' ns_search_comb(record_subtype = "macrogroup")
#' ns_search_comb(modified_since = "2020-04-30T00:00:00+0000")
#' ns_search_comb(page = 0, per_page = 2)
#' }
ns_search_comb <- function(text = NULL, text_adv = NULL, status = NULL,
  location = NULL, record_type = NULL, record_subtype = NULL,
  modified_since = NULL, page = NULL, per_page = NULL, ...) {

  paging <- handle_paging(page, per_page)
  text <- handle_text(text, text_adv)
  status <- handle_status(status)
  location <- handle_location(location)
  record_type <- handle_type(record_type)
  record_subtype <- handle_subtype(record_subtype)
  assert(modified_since, "character")
  res <- ns_POST(
    url = file.path(ns_base(), 'api/data/search'),
    body = list(criteriaType = "combined",
      textCriteria = list(text) %|lst|% NULL,
      statusCriteria = list(status) %|lst|% NULL,
      locationCriteria = list(location) %|lst|% NULL,
      recordTypeCriteria = list(record_type) %|lst|% NULL,
      recordSubtypeCriteria = list(record_subtype) %|lst|% NULL,
      pagingOptions = paging,
      modifiedSince = modified_since
    ),
    ...
  )
  parse_search(res)
}

#' NatureServe taxonomic name search
#'
#' @export
#' @param x (character) A name to search for. An asterisk (*) wildcarded species
#' name, e.g., 'Aquila chry*'. Name matching is case-insensitive and all of
#' the primary and synonymous scientific names, along with all common names,
#' are matched. Required.
#' @template ns
#'
#' @return A tibble (data.frame), with columns:
#' \itemize{
#'  \item jurisdictionScientificName - Scientfic name
#'  \item commonName - Common name
#'  \item globalSpeciesUid - UID - the taxonomic identifier NatureServe uses
#'  \item natureServeExplorerURI - URL to get to info online for the taxon
#'  \item taxonomicComments - comments about the taxon, if any
#' }
#'
#' @examples \dontrun{
#' ns_search(x = "Ruby*")
#' ns_search(x = "Helianthus annuus")
#' ns_search(x = "Ursus americanus")
#' }
ns_search <- function(x, key = NULL, ...) {
  assert(x, "character")
  res <- ns_GET(
    url = paste0(ns_base(), '/v1/globalSpecies/list/nameSearch'),
    query = list(name = x, NSAccessKeyId = check_key(key)),
    err_fxn = err_catch_search,
    ...
  )
  xml <- xml2::read_xml(res)
  kids <- xml2::xml_children(xml2::xml_children(xml)[[2]])
  dat <- lapply(kids, function(z) {
    data.frame(sapply(xml2::xml_children(z), function(x) {
      as.list(stats::setNames(xml2::xml_text(x), xml2::xml_name(x)))
    }), stringsAsFactors = FALSE)
  })
  df <- data.table::setDF(data.table::rbindlist(dat, use.names = TRUE, fill = TRUE))
  df <- move_col2(df, "natureServeExplorerURI")
  tibble::as_data_frame(df)
}

context("ns_search")

test_that("ns_search works as expected", {
  skip_on_cran()

  vcr::use_cassette("ns_search", {
    aa <- ns_search(x = "Helianthus annuus")
  })

  expect_is(aa, 'tbl_df')
  expect_is(aa$globalSpeciesUid, 'character')
  expect_is(aa$jurisdictionScientificName, 'character')
  expect_is(aa$commonName, 'character')
  expect_is(aa$natureServeExplorerURI, 'character')

  expect_equal(aa$jurisdictionScientificName, 'Helianthus annuus')
})

test_that("ns_search fails well", {
  skip_on_cran()

  vcr::use_cassette("ns_search_error", {
    expect_error(ns_search("asdfasf"), 'no results found')
  })

  expect_error(ns_search(),
               'argument "x" is missing')

  # fails well when input not character
  expect_error(ns_search(5), 'x must be of class character')
  expect_error(ns_search(TRUE), 'x must be of class character')
})

#' NatureServe data
#'
#' @export
#' @template ns
#' @param uid (character) a species UID, e.g., ELEMENT_GLOBAL.2.100925
#' @template status
#'
#' @return a named list, with possible slots:
#' \itemize{
#'  \item natureserve_uri
#'  \item classification
#'  \item economicAttributes
#'  \item license
#'  \item references
#'  \item conservationStatus
#'  \item managementSummary
#'  \item distribution
#'  \item ecologyAndLifeHistory
#' }
#'
#' @examples \dontrun{
#' ## single id
#' (res <- ns_data(uid = 'ELEMENT_GLOBAL.2.100925'))
#' ## many ids at once
#' res <- ns_data(uid = c('ELEMENT_GLOBAL.2.100925', 'ELEMENT_GLOBAL.2.104470'))
#' res$ELEMENT_GLOBAL.2.100925
#' res$ELEMENT_GLOBAL.2.104470
#'
#' ns_data(uid = 'ELEMENT_GLOBAL.2.101998')
#' }
ns_data <- function(uid, key = NULL, ...) {
  assert(uid, "character")
  uid <- toupper(uid)
  check_uid(uid)

  res <- ns_GET(
    url = paste0(ns_base(), '/v1.1/globalSpecies/comprehensive'),
    query = list(
      uid = paste0(uid, collapse = ","),
      NSAccessKeyId = check_key(key)),
    err_fxn = err_catch_data,
    ...
  )
  xml <- xml2::read_xml(res, encoding = "UTF-8")
  xml <- xml2::xml_children(xml)
  stats::setNames(lapply(xml, function(m) {
    nsuri <- xml2::xml_text(xml2::xml_find_first(m, ".//d1:natureServeExplorerURI"))
    class <- xml2::as_list(xml2::xml_find_first(m, ".//d1:classification"))
    ecostat <- xml2::xml_text(xml2::xml_find_first(m, ".//d1:economicAttributes"))
    ecology <- xml2::as_list(xml2::xml_find_first(m, ".//d1:ecologyAndLifeHistory"))
    license <- strtrim(xml2::xml_text(xml2::xml_find_first(m, ".//d1:license")))
    refs <- xml2::xml_text(xml2::xml_children(xml2::xml_find_first(m, "//d1:references")))
    constat <- xml2::xml_find_first(m, ".//d1:conservationStatus")
    constat_c <- xml2::xml_children(xml2::xml_find_first(constat, './/d1:otherStatuses'))
    constat_other <- stats::setNames(lapply(constat_c, function(z) {
      sapply(xml2::xml_children(z), function(x)
        as.list(stats::setNames(xml2::xml_text(x), xml2::xml_name(x))))
    }), xml2::xml_attr(constat_c, "name"))
    constat_ns <- parse_nss(xml2::xml_find_first(constat, './/d1:natureServeStatus'))
    ms <- as_list_(xml2::xml_find_first(m, ".//d1:managementSummary"))
    dist <- parse_dist(xml2::xml_find_first(m, ".//d1:distribution"))
    list(
      uid = xml2::xml_attr(m, "uid"),
      speciesCode = xml2::xml_attr(m, "speciesCode"),
      natureserve_uri = nsuri,
      classification = class,
      economicAttributes = ecostat,
      ecologyAndLifeHistory = ecology,
      license = license,
      references = refs,
      conservationStatus = list(other = constat_other, natureserve = constat_ns),
      managementSummary = ms,
      distribution = dist
    )
  }), uid)
}

# parsers
parse_dist <- function(x) {
  tmp <- xml2::xml_children(x)
  list(
    conservationStatusMap = xml2::xml_text(which_name(tmp, "conservationStatusMap")),
    globalRange = parse_if_1(tmp, "globalRange"),
    rangeMap = xml2::xml_text(which_name(tmp, "rangeMap")),
    endemism = parse_if_1(tmp, "endemism"),
    nations = {
      ch <- xml2::xml_children(which_name(tmp, "nations"))
      stats::setNames(lapply(ch, function(z) {
        if (xml2::xml_length(z) == 0) {
          list()
        } else {
          list(
            nationalDistributions = {
              as_list_(xml2::xml_child(which_name(xml2::xml_children(z), "nationalDistributions")[[1]]))
            },
            subnations = {
              sbn <- which_name(xml2::xml_children(z), "subnations")
              if (length(sbn) == 0) return(list())
              kids <- xml2::xml_children(sbn[[1]])
              lapply(kids, function(z) {
                c(
                  as.list(xml2::xml_attrs(z)),
                  as_list_(xml2::xml_find_first(z, "//d1:subnationalDistribution"))
                )
              })
            }
          )
        }
      }), xml2::xml_attr(ch, "nationCode"))
    },
    watersheds = {
      ch <- xml2::xml_children(which_name(tmp, "watersheds"))
      tibble::as_data_frame(data.table::rbindlist(lapply(xml2::xml_children(ch), function(w) {
        list(
          type = xml2::xml_attr(w, "type"),
          watershedName = xml2::xml_text(xml2::xml_find_first(w, "d1:watershedName")),
          watershedCode = xml2::xml_text(xml2::xml_find_first(w, ".//d1:watershedCode")),
          speciesOccurrenceCount = xml2::xml_text(xml2::xml_find_first(w, "d1:speciesOccurrenceCount"))
        )
      }), use.names = TRUE, fill = TRUE))
    },
    countyDistribution = {
      ch <- xml2::xml_children(xml2::xml_find_first(which_name(tmp, "countyDistribution"), "d1:occurrenceNations"))
      stats::setNames(lapply(ch, function(z) {
        st <- xml2::xml_children(xml2::xml_find_first(z, "d1:occurrenceStates"))
        stats::setNames(lapply(st, function(g) {
          data.table::setDF(data.table::rbindlist(
            lapply(xml2::xml_children(xml2::xml_find_first(g, "d1:occurrenceCounties")), as_list_),
            fill = TRUE,
            use.names = TRUE
          ))
        }), xml2::xml_attr(st, "code"))
      }), xml2::xml_attr(ch, "code"))
    }
  )
}

parse_nss <- function(x) {
  tmp <- xml2::xml_children(xml2::xml_find_first(x, "d1:globalStatus"))
  list(
    rank = xml2::xml_text(which_name(tmp, "rank")),
    roundedRank = xml2::as_list(which_name(tmp, "roundedRank")[[1]]),
    statusLastReviewed = xml2::xml_text(which_name(tmp, "statusLastReviewed")),
    statusLastChanged = xml2::xml_text(which_name(tmp, "statusLastChanged")),
    reasons = xml2::xml_text(which_name(tmp, "reasons")),
    conservationStatusFactors = {
      ch <- xml2::xml_children(which_name(tmp, "conservationStatusFactors"))
      if (length(ch) == 0) {
        list()
      } else {
        list(
          globalAbundance = parse_if_1(ch, "globalAbundance"),
          estimatedNumberOfOccurrences = parse_if_1(ch, "estimatedNumberOfOccurrences"),
          globalShortTermTrend = xml2::xml_text(which_name(ch, "globalShortTermTrend")),
          globalLongTermTrend = xml2::xml_text(which_name(ch, "globalLongTermTrend")),
          globalProtection = parse_if_1(ch, "globalProtection"),
          threat = xml2::xml_text(which_name(ch, "threat"))
        )
      }
    },
    nationalStatuses = {
      ch <- xml2::xml_children(which_name(tmp, "nationalStatuses")[[1]])
      stats::setNames(lapply(ch, function(w) {
        w <- xml2::xml_children(w)
        list(
          rank = xml2::xml_text(which_name(w, "rank")),
          roundedRank = xml2::xml_text(which_name(w, "roundedRank")),
          statusLastReviewed = xml2::xml_text(which_name(w, "statusLastReviewed")),
          subnationalStatuses = {
            subst <- xml2::xml_children(which_name(w, "subnationalStatuses"))
            lapply(subst, function(z) {
              c(
                as.list(xml2::xml_attrs(z)),
                rank = xml2::xml_text(which_name(xml2::xml_children(z), "rank")),
                roundedRank = xml2::xml_text(which_name(xml2::xml_children(z), "roundedRank"))
              )
            })
          }
        )
      }), xml2::xml_attr(ch, "nationCode"))
    }
  )
}

context("ns_data")

test_that("ns_data works as expected", {
  skip_on_cran()

  vcr::use_cassette("ns_data", {
    aa <- ns_data(uid = 'ELEMENT_GLOBAL.2.100925')
  })

  expect_is(aa, 'list')
  expect_is(aa[[1]], 'list')
  expect_is(aa[[1]]$distribution, 'list')
  expect_is(aa[[1]]$distribution$countyDistribution$US$AZ, 'data.frame')

  expect_match(aa[[1]]$natureserve_uri, 'explorer.natureserve.org')
})

test_that("ns_data eg works that didn't used to work", {
  skip_on_cran()

  # this didn't work cause Canada didn't have subnation data,
  # but should be fixed now
  vcr::use_cassette("ns_data_fixed", {
    aa <- ns_data(uid = 'ELEMENT_GLOBAL.2.101998')
  })

  expect_is(aa, 'list')
  expect_is(aa[[1]], 'list')
  expect_is(aa[[1]]$distribution, 'list')
  expect_equal(length(aa$ELEMENT_GLOBAL.2.101998$distribution$nations$CA), 0)
  expect_gt(length(aa$ELEMENT_GLOBAL.2.101998$distribution$nations$US$subnations), 0)
})

test_that("ns_data fails well", {
  skip_on_cran()

  expect_error(ns_data(),
               'argument "uid" is missing')
  expect_error(ns_data("asdfasf"),
               "one or more 'uid's")

  # fails well when input not character
  expect_error(ns_data(5), 'uid must be of class character')
  expect_error(ns_data(TRUE), 'uid must be of class character')
})

#' Get taxon by uid
#'
#' @export
#' @param uid (character) A NatureServe taxon id (The taxon’s Element Global
#' UID). required.
#' @template ns
#' @return A list with lots of elements
#' @details see https://explorer.natureserve.org/api-docs/#_taxon_data_model
#' for details on the response data
#' @examples \dontrun{
#' ns_id("ELEMENT_GLOBAL.2.154701")
#' }
ns_id <- function(uid, ...) {
  assert(uid, "character")
  stopifnot("`uid` must be length 1" = length(uid) == 1)
  res <- ns_GET(
    url = file.path(ns_base(), 'api/data/taxon', uid),
    ...
  )
  jsonlite::fromJSON(res)
}

test_that("ns_id", {
  skip_on_cran()

  vcr::use_cassette("ns_id", {
    aa <- ns_id("ELEMENT_GLOBAL.2.154701")
  })

  expect_is(aa, 'list')
  expect_is(aa$iucn, 'list')
  expect_equal(aa$iucn$iucnCode, 'VU')
  expect_is(aa$scientificName, 'character')
  expect_equal(aa$scientificName, 'Hydrastis canadensis')
  expect_is(aa$elementNationals, 'data.frame')
  expect_is(aa$speciesCharacteristics, 'list')
})

test_that("ns_id fails well", {
  skip_on_cran()

  # uid required
  expect_error(ns_id())
  # type wrong
  expect_error(ns_id(5))
  # uid must be length 1
  expect_error(ns_id(letters[1:3]))
})

ns_base <- function() 'https://explorer.natureserve.org'

ns_GET <- function(url, query = list(), ...){
  cli <- crul::HttpClient$new(url = url, opts = list(...))
  temp <- cli$get(query = query)
  temp$raise_for_status()
  x <- temp$parse("UTF-8")
  return(x)
}

ns_POST <- function(url, body, query = list(), ...){
  cli <- crul::HttpClient$new(url = url, opts = list(...),
    headers = list(Accept = "application/json"))
  temp <- cli$post(body = body, query = query, encode = "json")
  temp$raise_for_status()
  x <- temp$parse("UTF-8")
  return(x)
}

if (base::getRversion() >= "2.15.1") {
  utils::globalVariables(
    c("currentPresenceAbsence", "element_blank", "group", "lat", "long",
    "speciesOccurrenceCount", "theme", "unit", "nat_states")
  )
}

#' @title natserv
#' @description Interface to NatureServe https://www.natureserve.org/
#' @name natserv-package
#' @importFrom jsonlite fromJSON
#' @importFrom crul HttpClient
#' @importFrom tibble as_tibble enframe
#' @aliases natserv
#' @docType package
#' @author Scott Chamberlain \email{myrmecocystus@@gmail.com}
#' @keywords package
NULL

#' A data.frame with 49 rows and 2 columns
#'
#' - state (character) state 2 letter abbreviation
#' - state_name (character) state full name
#'
#' @name nat_states
#' @docType data
#' @keywords data
NULL

#' Search exports
#'
#' @export
#' @param text (character) basic text search, equiavalent to `text_adv`
#' with `matchAgainst="allNames"` and `operator="similarTo"`
#' @param text_adv (list) advanced search, must specify the following three
#' elements: `searchToken`, `matchAgainst`, and `operator`. see 
#' https://explorer.natureserve.org/api-docs/#_advanced_text_search_parameter
#' @param status (character) conservation status, one of G1, G2, G3, G4,
#' G5, GH, GX, GNR, GNA, GU. case insensitive
#' @param location (list) location, country and sub-country. specify either
#' `nation` OR `nation` and `subnation`. each expects a two-letter ISO code
#' @param record_type (character) limit results by record type, one of
#' "species" or "ecosystem"
#' @param record_subtype (character) limit results by record sub-type, one of:
#' "class", "subclass", "formation", "division", "macrogroup", "group",
#' "alliance", "association", "terrestrial_ecological_system"
#' @param modified_since (character) search for records modified since a
#' given time. value must be a date and time with a UTC offset in ISO 8601
#' format. optional
#' @param format (character) output format, one of "json" or "xlsx"
#' @param lang (character) language, one of "en", "es", or "fr"
#' @param id (character) a job id, from output of `ns_export()`
#' @template ns
#' @return `ns_export()` returns a single character string (a job id)
#' `ns_export_status()` returns a list of metadata concerning the 
#' status of the export
#' @examples \dontrun{
#' x <- ns_export(text = "robin")
#' res <- ns_export_status(x)
#' str(res)
#' res$state
#' res$data$errorMessage
#' res$data$url
#' 
#' w <- ns_export(text_adv = list(searchToken = "western",
#'   matchAgainst="allScientificNames", operator="startsWith"))
#' m <- ns_export_status(w)
#' head(jsonlite::fromJSON(m$data$url))
#' }
ns_export <- function(text = NULL, text_adv = NULL, status = NULL,
  location = NULL, record_type = NULL, record_subtype = NULL,
  modified_since = NULL, format = "json", lang = "en", ...) {

  text <- handle_text(text, text_adv)
  status <- handle_status(status)
  location <- handle_location(location)
  record_type <- handle_type(record_type)
  record_subtype <- handle_subtype(record_subtype)
  assert(format, "character")
  stopifnot(format %in% c("json", "xlsx"))
  format <- paste0("summary_", format)
  assert(lang, "character")
  stopifnot(tolower(lang) %in% c("en", "es", "fr"))
  assert(modified_since, "character")
  res <- ns_POST(
    url = file.path(ns_base(), 'api/export/taxonSearch'),
    body = list(
      criteria = tc(list(
        criteriaType = "combined",
        textCriteria = list(text) %|lst|% list(),
        statusCriteria = list(status) %|lst|% list(),
        locationCriteria = list(location) %|lst|% list(),
        recordTypeCriteria = list(record_type) %|lst|% list(),
        recordSubtypeCriteria = list(record_subtype) %|lst|% list(),
        modifiedSince = modified_since %||% NULL
      )),
      exportFormat = format,
      exportLanguage = lang
    ),
    ...
  )
  jsonlite::fromJSON(res)$jobId
}

#' @export
#' @rdname ns_export
ns_export_status <- function(id, ...) {
  assert(id, "character")
  res <- ns_GET(file.path(ns_base(), 'api/job', id), ...)
  jsonlite::fromJSON(res)
}

test_that("ns_export", {
  skip_on_cran()

  vcr::use_cassette("ns_export", {
    aa <- ns_export(text = "tanager")
  })

  expect_is(aa, 'character')
  expect_match(aa, '[0-9a-z].+')
  expect_gt(nchar(aa), 30)

  vcr::use_cassette("ns_export_status", {
    res <- ns_export_status(aa)
  })

  expect_is(res, 'list')
  expect_named(res)
  expect_is(res$state, 'character')
  expect_is(res$data$url, 'character')
})

test_that("ns_export fails well", {
  skip_on_cran()

  # text must be character
  expect_error(ns_export(text = 5))
  # text_adv must be character
  expect_error(ns_export(text_adv = 5))
  # status must be character, and in allowed set
  expect_error(ns_export(status = 5))
  expect_error(ns_export(status = "foo"))
  # location must be NULL or a list
  expect_error(ns_export(location = 5))
  expect_error(ns_export(location = list(foobar = "US")))
  expect_error(ns_export(location = list(nation = "US", dub = "d")))
  # record_type must be character, and in allowed set
  expect_error(ns_export(record_type = 5))
  expect_error(ns_export(record_type = "bear"))
  # record_subtype must be character, and in allowed set
  expect_error(ns_export(record_subtype = 5))
  expect_error(ns_export(record_subtype = "bear"))
  # modified_since must be character
  expect_error(ns_export(modified_since = 5))
})

#' Get taxon by uid, id, or elCode
#'
#' @export
#' @param uid (character) A NatureServe taxon id (The taxon’s Element Global
#' UID)
#' @param id The primary key value (ELEMENT_GLOBAL_ID) of the record within
#' Central Biotics
#' @param el_code The Biotics Element Code (ELCODE_BCD) of the record
#' @template ns
#' @return A list with lots of elements
#' @details see https://explorer.natureserve.org/api-docs/#_taxon_data_model
#' for details on the response data
#' @examples \dontrun{
#' ns_altid(uid = "ELEMENT_GLOBAL.2.154701")
#' ns_altid(id = "154701")
#' ns_altid(el_code = "PDRAN0F010")
#' }
ns_altid <- function(uid = NULL, id = NULL, el_code = NULL, ...) {
  assert(uid, "character")
  assert(id, "character")
  assert(el_code, "character")
  if (is.null(c(uid, id, el_code)))
    stop("one of `uid`, `id`, or `el_code` must be provided", call. = FALSE)
  if (length(c(uid, id, el_code)) > 1)
    stop("only one of `uid`, `id`, `el_code` can be provided", call. = FALSE)
  res <- ns_GET(
    url = file.path(ns_base(), 'api/data/taxonSearch'),
    query = tc(list(ouSeqUid = uid, id = id, elCode = el_code)),
    ...
  )
  jsonlite::fromJSON(res)
}

test_that("ns_altid", {
  skip_on_cran()

  vcr::use_cassette("ns_altid", {
    aa <- ns_altid(id = "154701")
  })

  expect_is(aa, 'list')
  expect_is(aa$iucn, 'list')
  expect_equal(aa$iucn$iucnCode, 'VU')
  expect_is(aa$scientificName, 'character')
  expect_equal(aa$scientificName, 'Hydrastis canadensis')
  expect_is(aa$elementNationals, 'data.frame')
  expect_is(aa$speciesCharacteristics, 'list')
})

test_that("ns_altid fails well", {
  skip_on_cran()

  # an id must be given
  expect_error(ns_altid())
  # only 1 id can be given
  expect_error(ns_altid("a", "b"))
  # type wrong
  expect_error(ns_altid(5))
  # uid must be length 1
  expect_error(ns_altid(letters[1:3]))
})

#' Species search
#'
#' @export
#' @inheritParams ns_search_comb
#' @param species_taxonomy (list) species taxonomy. either a list with
#' `level` and `scientificTaxonomy` (a scientific name), or with just
#' `informalTaxonomy` (a vernacular name). possible `level` values:
#' "kingdom", "phylum", "class", "order", "family", "genus" 
#' @template ns
#' @family search
#' @examples \dontrun{
#' ns_search_spp(text = "robin")
#' ns_search_spp(text_adv = list(searchToken = "bird", 
#'   matchAgainst = "allNames", operator="similarTo"))
#' ns_search_spp(status = "G1")
#' ns_search_spp(location = list(nation = "US"))
#' ns_search_spp(location = list(nation = "US", subnation = "VA"))
#' ns_search_spp(species_taxonomy = list(scientificTaxonomy = "Animalia", level = "kingdom"))
#' ns_search_spp(species_taxonomy = list(informalTaxonomy = "birds"))
#' ns_search_spp(record_subtype = "macrogroup")
#' ns_search_spp(modified_since = "2020-04-30T00:00:00+0000")
#' ns_search_spp(page = 0, per_page = 2)
#' }
ns_search_spp <- function(text = NULL, text_adv = NULL, status = NULL,
  location = NULL, species_taxonomy = NULL, record_subtype = NULL,
  modified_since = NULL, page = NULL, per_page = NULL, ...) {

  paging <- handle_paging(page, per_page)
  text <- handle_text(text, text_adv)
  status <- handle_status(status)
  location <- handle_location(location)
  species_taxonomy <- handle_sptax(species_taxonomy)
  record_subtype <- handle_subtype(record_subtype)
  assert(modified_since, "character")
  res <- ns_POST(
    url = file.path(ns_base(), 'api/data/speciesSearch'),
    body = list(criteriaType = "species",
      textCriteria = list(text) %|lst|% NULL,
      statusCriteria = list(status) %|lst|% NULL,
      locationCriteria = list(location) %|lst|% NULL,
      speciesTaxonomyCriteria = list(species_taxonomy) %|lst|% NULL,
      recordSubtypeCriteria = list(record_subtype) %|lst|% NULL,
      pagingOptions = paging,
      modifiedSince = modified_since
    ),
    ...
  )
  parse_search(res)
}

test_that("ns_search_spp", {
  skip_on_cran()

  vcr::use_cassette("ns_search_spp", {
    aa <- ns_search_spp(text = "robin")
  })

  expect_is(aa, 'list')
  expect_is(aa$results, 'data.frame')
  expect_is(aa$results, 'tbl_df')
  expect_is(aa$resultsSummary, 'data.frame')
  expect_is(attr(aa$resultsSummary, "search_criteria"), 'list')
})

test_that("ns_search_spp fails well", {
  skip_on_cran()

  # text must be character
  expect_error(ns_search_spp(text = 5))
  # text_adv must be character
  expect_error(ns_search_spp(text_adv = 5))
  # status must be character, and in allowed set
  expect_error(ns_search_spp(status = 5))
  expect_error(ns_search_spp(status = "foo"))
  # location must be NULL or a list
  expect_error(ns_search_spp(location = 5))
  expect_error(ns_search_spp(location = list(foobar = "US")))
  expect_error(ns_search_spp(location = list(nation = "US", dub = "d")))
  # species_taxonomy must be NULL or a list, and in allowed set
  expect_error(ns_search_spp(species_taxonomy = 5))
  expect_error(ns_search_spp(species_taxonomy = list(foobar = "US")))
  expect_error(ns_search_spp(species_taxonomy = list(foo = 3, level = 1)))
  # record_subtype must be character, and in allowed set
  expect_error(ns_search_spp(record_subtype = 5))
  expect_error(ns_search_spp(record_subtype = "bear"))
  # modified_since must be character
  expect_error(ns_search_spp(modified_since = 5))
  # paging must be a list, and both must be set if any given
  expect_error(ns_search_spp(page = "d"))
  expect_error(ns_search_spp(per_page = "d"))
  expect_error(ns_search_spp(page = 5))
  expect_error(ns_search_spp(per_page = 5))
})

handle_paging <- function(page, per_page) {
  assert(page, c('numeric', 'integer'))
  assert(per_page, c('numeric', 'integer'))
  if (!is.null(page) || !is.null(per_page)) {
    if (length(c(page, per_page)) != 2)
      stop("if `page` given, `per_page` must be given, and vice versa",
        call. = FALSE)
  }
  paging <- tcnull(list(page = page, recordsPerPage = per_page))
  return(paging)
}
handle_text <- function(text, text_adv) {
  assert(text, "character")
  assert(text_adv, "list")
  if (length(tc(list(text, text_adv))) > 0)
    stopifnot(xor(is.null(text), is.null(text_adv)))
  if (!is.null(text)) text <- list(paramType="quickSearch", searchToken=text)
  if (!is.null(text_adv)) {
    txt_adv_names = c("searchToken", "matchAgainst", "operator")
    assert(text_adv, "list")
    if (!all(names(text_adv) %in% txt_adv_names)) {
      stop("text_adv format incorrect; see ns_search_comb docs",
        call. = FALSE)
    }
    text <- c(paramType="textSearch", text_adv)
  }
  return(text)
}
p <- function(x) paste0(x, collapse = ", ")
handle_status <- function(status) {
  assert(status, "character")
  if (!is.null(status)) {
    z <- c("G1", "G2", "G3", "G4", "G5", "GH", "GX", "GNR", "GNA", "GU")
    if (!tolower(status) %in% tolower(z))
      stop("`status` must be one of ", p(z), call. = FALSE)
    status <- list(paramType="globalRank", globalRank=status)
  }
  return(status)
}
handle_location <- function(location) {
  assert(location, "list")
  if (!is.null(location)) {
    if (!all(names(location) %in% c("nation", "subnation"))) {
      stop("`location` must be a list w/ 'nation' or 'nation' and 'subnation'",
        call. = FALSE)
    }
    pt <- if ("subnation" %in% names(location)) "subnation" else "nation"
    location <- c(paramType=pt, location)
  }
  return(location)
}
handle_type <- function(record_type) {
  assert(record_type, "character")
  if (!is.null(record_type)) {
    z <- c("species", "ecosystem")
    if (!tolower(record_type) %in% z)
      stop("`record_type` must be one of ", p(z),call. = FALSE)
    record_type <- list(paramType="recordType", recordType=record_type)
  }
  return(record_type)
}
handle_subtype <- function(record_subtype) {
  assert(record_subtype, "character")
  if (!is.null(record_subtype)) {
    z <- c("class", "subclass", "formation", "division", "macrogroup", "group",
      "alliance", "association", "terrestrial_ecological_system")
    if (!tolower(record_subtype) %in% z)
      stop("`record_subtype` must be one of ", p(z),call. = FALSE)
    return(list(paramType="ecosystemType", ecosystemType=record_subtype))
  }
  return(record_subtype)
}
handle_sptax <- function(st) {
  assert(st, "list")
  if (!is.null(st)) {
    if (
      !all(names(st) %in% 
        c("level", "scientificTaxonomy", "informalTaxonomy"))
    ) {
      stop("`species_taxonomy` must be a list w/ 'informalTaxonomy' ",
        "or 'level' and 'scientificTaxonomy'",
        call. = FALSE)
    }
    pt <- if ("informalTaxonomy" %in% names(st)) 
      "informalTaxonomy" 
    else
      "scientificTaxonomy"
    st <- c(paramType=pt, st)
  }
  return(st)
}
handle_ecotax <- function(x) {
  assert(x, "character")
  if (!is.null(x)) {
    return(list(paramType="ecosystemHierarchyAncestor", classificationCode=x))
  }
  return(x)
}

parse_search <- function(x) {
  tt <- jsonlite::fromJSON(x)
  tt$results <- tibble::as_tibble(tt$results)
  tt$resultsSummary <- ns_sum(tt$resultsSummary)
  attr(tt$resultsSummary, "search_criteria") <- tt$searchCriteria
  tt$searchCriteria <- NULL
  return(tt)
}

ns_sum <- function(x) {
  x$species_total <- x$speciesResults$total
  x$speciesResults <- NULL
  eco <- x$ecosystemResults
  x$ecosystemResults <- NULL
  data.frame(tibble::enframe(c(x, eco)))
}

test_that("ns_ecohier", {
  skip_on_cran()

  vcr::use_cassette("ns_ecohier", {
    aa <- ns_ecohier("ELEMENT_GLOBAL.2.683060")
  })

  expect_is(aa, 'data.frame')
  expect_is(aa$uniqueId, 'character')
  expect_is(aa$name, 'character')
  expect_is(aa$nsxUrl, 'character')
  expect_is(aa$ecosystemType, 'character')
  expect_is(aa$classificationCode, 'character')
})

test_that("ns_ecohier fails well", {
  skip_on_cran()

  # an id must be given
  expect_error(ns_ecohier())
  # only 1 id can be given
  expect_error(ns_ecohier("a", "b"))
  # type wrong
  expect_error(ns_ecohier(5))
})

<!--
%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{Introduction to `natserve` - An R package to wrap NatureServe's database API from rOpenSci}
%\VignetteEncoding{UTF-8}
-->

```{r echo=FALSE}
knitr::opts_chunk$set(
	comment = "#>",
	collapse = TRUE,
	warning = FALSE,
	message = FALSE
)
```

=====

`natserv` is an R package that interacts with the API services of the non-profit organization [NatureServe](https://services.natureserve.org/). If you want to read the full API documentation, you can find it on [NatureServe's web services webpage](https://services.natureserve.org/BrowseServices/getSpeciesData/getSpeciesListREST.jsp).

See also the [taxize book](https://ropensci.github.io/taxize-book/) for 
a manual on working with taxonomic data in R, including with NatureServe data.

This tutorial will walk you through installing `natserv` and using its handy functions. To help show you what `natserv` can do, this will also teach you how to:

* Find a species' unique identifier used by NatureServe 
* Use that identifier to pull data from NatureServe's API
* Search NatureServe for multiple species
* Create a dataframe of the conservation status of an example species, the gray wolf (*Canis lupus*)
* And, just for fun, we're going to make a map of the gray wolf's status in each state of the US with `ggplot`


## A quick introduction to NatureServe
NatureServe is a non-profit organization that provides biodiversity data freely online. 
They maintain a database comprised of data from natural heritage programs and conservation data centers - this database includes information about the conservation status, taxonomy, geographic distribution, and life history information for over 70,000 species of plants, animals, and fungi in Canada and the United States.
You can find information about their [data coverage](http://explorer.natureserve.org/summary.htm), [data types](http://explorer.natureserve.org/methods.htm), and [data sources](http://explorer.natureserve.org/sources.htm) on their website.
NatureServe also hosts data on ecological communities/systems and their conservation status.  

While small amounts of data can be easily collected using their online [NatureServe explorer site](http://explorer.natureserve.org/), downloading species data this way would be incredibly slow. 
Thus `natserv` was born. 
This R package can access NatureServe's online API for rapid downloading of conservation data, allows for easy access to multiple species' data sets, and  loads the data directly into your R session. 

## Installing `natserv` from CRAN or GitHub
The stable version of `natserv` is available on the [CRAN repository](https://CRAN.R-project.org/package=natserv). You can install it with:

```{r install.natserv, eval=F, error=F, message=F}
install.packages("natserv")
```

If you want the latest development version of `natserv`, you can install the package from the ropensci GitHub repository. Do this by first installing the `devtools` library, then installing directly from GitHub:

```{r install.natserv.github, message=F, error=F}
install.packages("devtools")
devtools::install_github("ropensci/natserv")
```

After successful installation, load the package into the environment:
```{r library.natserv, message=F, error=F}
library(natserv)
```

## Obtaining a NatureServe API key

Before starting to use the NatureServe API, you need to have an API key.
NatureServe uses this key to track and prevent any malicious usage and the `natserv` package needs it in order to access their database.

Fortunately, API keys are available for free to users who register with NatureServe as Developers.
To register and obtain an API key, go to this link: [https://services.natureserve.org/developer/index.jsp](https://services.natureserve.org/developer/index.jsp).

After registering, NatureServe will send you an email at the registered address that contains an Access Key ID.

###  Linking R to your API key
There are two ways to do this - the one that works best for you likely depends on how often you plan on using `natserv`. 

#### 1. Define the API key just for this R session
The quick way is to load the API key directly in the R session:
```{r my.api.key, include=F}
options(NatureServeKey = "4c9cb1d7-c017-497c-8fe4-961f153cfc36")
```

```{r nat.serv.key, eval=F}
options(NatureServeKey = "YOUR-ACCESS-KEY-ID")
```
Every time you start a new R session and want to use `natserv`, you'll have to run this line.


#### 2. Define the API key for all scripts in your project

If you don't want to run `options(NatureServeKey = "YOUR-ACCESS-KEY-ID")` every time you use `natserv`, you can modify the `.Rprofile` or `.Renviron` files to link the API key for all R scripts in your Rproject.
When R starts up, it reads the `.Rprofile` and `.Renviron` and runs any commands in these files as it launches. 

To set the API key for all scripts in the project, open your project-specific files with `file.edit(".Rprofile")` or `file.edit(".Renviron")`, then add this line:

```
options(NatureServeKey = "YOUR-ACCESS-KEY-ID")
```

After saving this file, every time you open an R session in that project your API key is automatically loaded into the session. 

## `natserv` functions

Now that R knows our NatureServe API key, we can use `natserv` to access their database.
All of `natserv`'s functions are prefixed with `ns_` to avoid confusion with other packages - here are the functions provided by `natserv`:

1. `ns_search` - a function to look up the ID code for the species you are querying
2. `ns_data` - a function that uses the ID code from `ns_search` to pull data off of NatureServe and into your R session
3. `ns_images` - a function that searches for metadata for the images hosted by NatureServe, including the URLs where the images can be found

### `ns_search` - finding the UID
NatureServe uses a unique ID (UID) for each species in their database. 
The `ns_data` function requires you to supply the UID for the species you are looking to download data for, and unless you already know the species' UID you will need to use `ns_search` to find it. 

Here's how to use a species' binomial name to find its UID, using the gray wolf (*Canis lupus*) as an example:

```{r ns.search.sciname}
gray.wolf.search <- ns_search(x="Canis lupus")
gray.wolf.search
```
After a moment of searching, `ns_search` returns a tibble containing the UID, common name, scientific name, taxonomic comments, and the URI for the entry in the NatureServe database.

```{r uid.return}
gray.wolf.search$globalSpeciesUid
```

The UID for *Canis lupus* is `ELEMENT_GLOBAL.2.105212`, and this ID can be used to access the NatureServe database through their API. 

#### A quick aside: Searching for multiple species

**Searching for all species within a genus**

If we are interested in multiple species (all members of the genus *Canis* for example), we can use `*` as a wild card to pull out all available UIDs for a genus:

```{r ns.search.wildcard}
canis.search <- ns_search(x="Canis *")
```

This returns a tibble with 14 rows, one for each entry in *Canis*: 

```{r, echo=F}
canis.search
```

**Searching for multiple species if the wildcard won't do**

If the species you want data about aren't members of the same genus, the wild card method won't work. 
Instead, to search for multiple taxa you can make a vector of search terms containing the scientific names of each species you're interested in, then search each one in a loop or with the `lapply` function:

```{r ns.search.multisp}
species.search.list <- c('Canis lupus', 'Lynx rufus', 'Puma concolor')
multispecies.search <- lapply(species.search.list, FUN = ns_search)
```

This returns a 3-element list, where each list item is the tribble returned by `ns_search`:

```{r multi.sp.search, echo=F}
multispecies.search
```

### `ns_data` - searching for records with the UID

Now that we have our species' UID, we can search NatureServe's API for more data about that species' conservation status, taxonomy, and other information. 

Sticking to the gray wolf example, let's pull up what data are available for *Canis lupus* using the UID from `ns_search`:

```{r ns.data}
gray.wolf.data <- ns_data(gray.wolf.search$globalSpeciesUid)
```

#### Data returned by `ns_data`

The `ns_data` command returns NatureServe data as a list of lists. 
The outer 'list' has a length of 1 and is named after the species' UID. Within this top-level list are more nested sublists with the following information (*note: this is not exhaustive; more data are available if you dig through the list structure*):

* `natureserve_uri` - the URI for the species' web page
* `classification` - a list containing taxonomic information for the species
  * `classification$names` - scientific name(s) -  either formatted (i.e. italicized) or not - with information about the author of the nomenclature, concept reference information, and classification status
  * `classification$natureServePrimaryGlobalCommonName` - the common name for the species used by NatureServe
  * `classification$otherGlobalCommonNames` - other common names with the language of that common name provided by `attr(,"language")
  * `classification$taxonomy` - information regarding kingdom, phylum, class, genus, and comments on the taxonomic classification of the species; both formal and informal taxonomies are provided
* `economicAttributes` - the economic impacts of the species and how it impacts human activities
* `license` - information about NatureServe's data usage license
* `references` - a list of scientific references that collected the reported data
* `conservationStatus` - a list containing conservation status information from a variety of geographic locations
  * `conservationStatus$other$USESA Status` - US Endangered Species Act status
  * `conservationStatus$other$COSEWIC Status` - Committee on the Status of Endangered Wildlife in Canada status
  * `conservationStatus$other$IUCN Status` - International Union for the Conservation of Nature status
  * `conservationStatus$other$CITES Protection Status` - Convention on International Trade of Endangered Species status
  * `conservationStatus$natureserve` - a list of species ranks as designated by NatureServe, as well as descriptions, rationale, population estimates, current threats, comments, and information on when the status was last reviewed. This list also contains state- and province-wide status for the species where it is found
* `managementSummary` - a list containing subslists `$restorationPotential` (a description of whether restoration is feasible for the species) and `$managementRequirements` (factors which are necessary to promote restoration of the species)
* `distribution` - information on the species' distribution globally and within Canada and the United States
  * `distribution$conservationStatusMap` - a URI to an online conservation status map hosted on NatureServe
  * `distribution$globalRange`- information on the total area inhabited by the species, as well as a description of this global distribution and how it may have changed over time
  * `distribution$rangeMap` - a URI to an online range map hosted on NatureServe
  * `distribution$endemism` - a description of the species' endemism
  * `distribution$nations` - a list of where the species can be found during what times of year (i.e. year-round, regularly occurring, etc.) as well as information on sub-national scales like state and provincial presence of the species
  * `distribution$watersheds` - a tibble of watersheds where the species has been found and when
  * `distribution$countyDistribution` - a tibble containing county-wide information about where and when the species has been observed in the United States
  
Fortunately, even though the list structure is rather complex, you can view the entire list hierarchy with `str()`, short for `structure`:

```{r list.str, eval=F}
str(gray.wolf.data)
```
```
## List of 1
##  $ ELEMENT_GLOBAL.2.105212:List of 11
##   ..$ uid                  : chr "ELEMENT_GLOBAL.2.105212"
##   ..$ speciesCode          : chr "AMAJA01030"
##   ..$ natureserve_uri      : chr "http://explorer.natureserve.org/servlet/NatureServe?searchName=Canis+lupus"
##   ..$ classification       :List of 2
##   .. ..$ names   :List of 3
##   .. .. ..$ scientificName                    :List of 4
##   .. .. .. ..$ unformattedName    :List of 1
##   .. .. .. .. ..$ : chr "Canis lupus"
##   .. .. .. ..$ formattedName      :List of 1
##   .. .. .. .. ..$ : chr "<i>Canis lupus</i>"
##   .. .. .. ..$ nomenclaturalAuthor:List of 1
##   .. .. .. .. ..$ : chr "Linnaeus, 1758"
##   .. .. .. ..$ conceptReference   :List of 3
##   .. .. .. .. ..$ formattedFullCitation     :List of 1
##   .. .. .. .. .. ..$ : chr "Wilson, D. E., and D. M. Reeder (editors). 1993. Mammal species of the world: a taxonomic and geographic refere"| __truncated__
##   .. .. .. .. ..$ nameUsedInConceptReference:List of 2
##   .. .. .. .. .. ..$ unformattedName:List of 1
##   .. .. .. .. .. .. ..$ : chr "Canis lupus"
##   .. .. .. .. .. ..$ formattedName  :List of 1
##   .. .. .. .. .. .. ..$ : chr "<i>Canis lupus</i>"
##   .. .. .. .. ..$ classificationStatus      :List of 1
##   .. .. .. .. .. ..$ : chr "Standard"
##   .. .. .. .. ..- attr(*, "code")= chr "B93WIL01NAUS"
##   .. .. ..$ natureServePrimaryGlobalCommonName:List of 1
##   .. .. .. ..$ : chr "Gray Wolf"
##   .. .. ..$ otherGlobalCommonNames            :List of 4
##   .. .. .. ..$ commonName:List of 1
##   .. .. .. .. ..$ : chr "Grey Wolf"
##   .. .. .. .. ..- attr(*, "language")= chr "EN"
##   .. .. .. ..$ commonName:List of 1
##   .. .. .. .. ..$ : chr "Lobo Gris"
##   .. .. .. .. ..- attr(*, "language")= chr "ES"
##   .. .. .. ..$ commonName:List of 1
##   .. .. .. .. ..$ : chr "Wolf"
##   .. .. .. .. ..- attr(*, "language")= chr "EN"
##   .. .. .. ..$ commonName:List of 1
##   .. .. .. .. ..$ : chr "loup gris"
##   .. .. .. .. ..- attr(*, "language")= chr "FR"
##   .. ..$ taxonomy:List of 2
##   .. .. ..$ formalTaxonomy  :List of 8
##   .. .. .. ..$ kingdom          :List of 1
##   .. .. .. .. ..$ : chr "Animalia"
##   .. .. .. ..$ phylum           :List of 1
##   .. .. .. .. ..$ : chr "Craniata"

... truncated to save space
```

To pull out a specific piece or type of information, it is necessary to dig through the lists of lists to get at what you want. 
For example, if I would like to know which Phylum the gray wolf belongs to, I first go into the top-level `ELEMENT_GLOBAL.2.105212` list, then into the `classification` list, then the `taxonomy` list, then `formalTaxonomy`, then `phylum` within  using `$` notation:

```{r ns.wolf.phylum}
gray.wolf.phylum <- gray.wolf.data$ELEMENT_GLOBAL.2.105212$classification$taxonomy$formalTaxonomy$phylum
gray.wolf.phylum
```

### `ns_images` - searching NatureServe's image repository

Unlike `ns_data`, the `ns_images` function can search for images of your species of interest using either the scientific name, UID, or common name of the species. 
`ns_images` can take one of three types of search information: `uid`, the UID of the species (returned by `ns_search`),  `scientificName`, or `commonName`: 

```{r ns.images}
# Search with the UID (ELEMENT_GLOBAL.2.105212)
gray.wolf.imageInfo <- ns_images(uid = 'ELEMENT_GLOBAL.2.105212')

# Or search using the scientific name:
gray.wolf.imageInfo <- ns_images(scientificName = 'Canis lupus')

# Or search using the common name:
gray.wolf.imageInfo <- ns_images(commonName = 'Gray wolf')
```

This returns metadata for images of the gray wolf hosted by NatureServe, including the creator, the publisher, the description, and usage rights. When no other arguments are given, `ns_images` returns metadata for all available images in their database. Later in the vignette, we'll look at how to return specific image resolutions according to your needs.

When we provide `ns_images` with only `scientificName` or `commonName` search names, it will only search for those terms in NatureServe's Primary Names field. 
But you can also search using synonymous common and scientific names for the species.
By including `includesSynonyms = 'Y'` in the `ns_images` command, the provided search term will be looked for in the Primary names of each species, as well as the synonymous Scientific and Common Names fields.
Of course, if you search using the UID (i.e. with `uid`), there are no synonyms. 

In the future it may also be possible to search using ITIS names using the `ITISNames` argument, but as of August 2018 searching with ITIS names is not possible.

An additional argument, `resolution`, can be used to indicate the desired resolution for the image. 
If `resolution` is not specified, `ns_images` returns metadata for all available image resolutions. 

To only return metadata for a certain `resolution`, you can supply one of the following values to the `resolution` argument:

* `resolution = 'lowest'` 
* `resolution = 'highest'` 
* `resolution = 'thumbnail'`
* `resolution = 'web'` (return metadata for the image used on NatureServe's web page for the species)

## Tutorial: Using `natserv` data to map wolf conservation status across the US

Now, using NatureServe data, let's see an example of what we can do with it. 
As an example to familiarize yourself with `natserv`, let's build a map of conservation status of the gray wolf in each state of the United States. 
We'll need to load a few libraries first (if you don't have them loaded already):

```{r plotting.libraries, message=F, warning=F, error=F}
library(natserv)
library(ggplot2)
library(dplyr)
```

Next, let's combine the `ns_search` and `ns_data` functions to pull out state-level conservation status for the gray wolf from NatureServe's API:

```{r wolf.conservation.df}
gray.wolf.alldata <- ns_data(ns_search('Canis lupus')$globalSpeciesUid)

# Now let's just pull out the sub-national conservation data
wolf.conservation.status <- gray.wolf.alldata$ELEMENT_GLOBAL.2.105212$conservationStatus$natureserve$nationalStatuses$US$subnationalStatuses

# This returns a list of lists containing the conservation status in each state
wolf.conservation.status[[1]]
```
```{r wolf.conservation.df.2}
# The outer list contains one entry for each state
# Let's convert it into a data.frame for mapping
wolf.conservation.df <- as.data.frame(matrix(unlist(wolf.conservation.status), nrow=length(wolf.conservation.status), byrow=T))
colnames(wolf.conservation.df) <- c('region','abbreviation','NatServ.Rank','NatServe.Rounded.Rank')

head(wolf.conservation.df)
```

NatureServe defines conservation status at the sub-national level (indicated with an `S` prefix - `N` indicates national-level status). Sub-national status is defined below (from the [NatureServe website](http://explorer.natureserve.org/nsranks.htm)):

```{r status.table, include=F}
library(knitr)
library(kableExtra)

status.table <- data.frame(Status = c('SX','SH','S1','S2','S3','S4', 'S5'), Definition=c('Presumed Extirpated: Species is believed to be extirpated from the state. Not located despite intensive searches of historical sites and other appropriate habitat, and virtually no likelihood that it will be rediscovered.', 'Possibly Extirpated (historical): Species or community occurred historically in the nation or state/province, and there is some possibility that it may be rediscovered. Its presence may not have been verified in the past 20-40 years.','Critically imperiled: Extremely rate (fewer than 5 occurances) or very steep declines make it very vulnerable to extirpation.','Imperiled: Rare due to restricted range, few populations (often fewer than 20) or other factors.','Vulnerable: Vulnerable to extirpation due to a restricted range, relatively few populations (often 80 or fewer), recent and widespread declines, or other factors','Apparently secure: Uncommon but not rare; some cause for long-term concern due to declines or other factors.','Secure: Common, widespread, and abundant'))

status.table %>% 
  kable() %>% 
  kable_styling(bootstrap_options = c('striped','hover'))
```

The other status definitions aren't included in the gray wolf dataset, and aren't described here. 

```{r status.rank.levels}
unique(wolf.conservation.df$NatServ.Rank)

# Check the order of levels in the NatServ.Rank factor column:
levels(wolf.conservation.df$NatServ.Rank)
```
At the moment, the levels of `NatServ.Rank` and `NatServe.Rounded.Rank` aren't ordered intuitively. 
Let's reorder them from most to least conservation concern. 
```{r relevel.status.factors}
wolf.conservation.df$NatServ.Rank <- factor(wolf.conservation.df$NatServ.Rank, 
levels(wolf.conservation.df$NatServ.Rank)[c(6,5,1,2,3,4)])

levels(wolf.conservation.df$NatServ.Rank)
```

Much better! Now let's use our data set to make a map. 

```{r plot.states}
# Using the ggplot2 library, we can automatically load map data for the US
states <- map_data('state')

# Let's first plot the empty map:
ggplot(data = states) + 
  geom_polygon(aes(x = long, y = lat, group = group), color = "black", fill="white") + 
  coord_fixed(1.3) +
  guides(fill=FALSE)  # this removes the legend from the map 
```

We now have to merge the two data sets - the `states` dataframe from `ggplot` and the NatureServe conservation status data. 
To do this, we can use the `inner_join` function from the `dplyr` package

```{r inner.join}
# Change wolf.conservation.df region to character, and coerce it to lowercase so that it matches the region name in the states dataframe
wolf.conservation.df$region <- tolower(as.character(wolf.conservation.df$region))
wolf.conservation.join <- full_join(states, wolf.conservation.df, by = 'region')

head(wolf.conservation.join)
```
And map it!

```{r status.map}
# Let's use the data to make a map
ggplot(data = wolf.conservation.join, aes(fill=NatServ.Rank)) + 
    # Fill in states that are NaN with white
  geom_polygon(data=states, aes(x = long, y = lat, group = group), color = 'black', fill='white') + 
  geom_polygon(aes(x = long, y = lat, group = group), color = "black") + 
  scale_fill_brewer(name='NatureServe Rank', palette = 'Spectral') + # Use the spectral palette
  coord_fixed(1.3)
```

---
title: "natserv introduction"
author: "Scott Chamberlain"
date: "2020-05-15"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{natserv introduction}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---



`natserv` is an R package that interacts with the API services of the non-profit organization NatureServe (https://services.natureserve.org/). If you want to read the full API documentation, you can find it at https://explorer.natureserve.org/api-docs/

See also the taxize book (https://taxize.dev/) for 
a manual on working with taxonomic data in R, including with NatureServe data.

This tutorial will walk you through installing `natserv` and using its functions. 

## A quick introduction to NatureServe

NatureServe is a non-profit organization that provides biodiversity data freely online. 
They maintain a database comprised of data from natural heritage programs and conservation data centers - this database includes information about the conservation status, taxonomy, geographic distribution, and life history information for over 70,000 species of plants, animals, and fungi in Canada and the United States.
You can find information about their data coverage (https://explorer.natureserve.org/AboutTheData/DataCoverage), and data sources (https://explorer.natureserve.org/AboutTheData/Sources) on their website. NatureServe also hosts data on ecological communities/systems and their conservation status.  

While small amounts of data can be easily collected using their online NatureServe explorer site (http://explorer.natureserve.org/), downloading species data this way would be incredibly slow. 
Thus `natserv` was born. 
This R package can access NatureServe's online API for rapid downloading of conservation data, allows for easy access to multiple species' data sets, and  loads the data directly into your R session. 

## Installing `natserv` from CRAN or GitHub

Stable version:


```r
install.packages("natserv")
```

Development version:


```r
remotes::install_github("ropensci/natserv")
```

After successful installation, load the package into the environment:


```r
library(natserv)
```

## `natserv` functions

All of `natserv`'s functions are prefixed with `ns_` to avoid confusion with other packages - here are the functions provided by `natserv`:


```r
cat(paste(" -", paste(sprintf("`%s`", sort(getNamespaceExports("natserv"))), collapse = "\n - ")))
#>  - `ns_altid`
#>  - `ns_ecohier`
#>  - `ns_export`
#>  - `ns_export_status`
#>  - `ns_id`
#>  - `ns_search_comb`
#>  - `ns_search_eco`
#>  - `ns_search_spp`
```

## Search

There's three functions for search:

- combined: supports searching for both species and ecosystems using search criteria which are applicable to both types of records
- species: supports searching for only species; extends the search criteria available through the Combined Search to include support for additional criteria which are only applicable to species records
- ecosystem: supports searching for only ecosystems; extends the search criteria available through the Combined Search to include support for additional criteria which are only applicable to ecosystems records

Combined search


```r
ns_search_comb(text = "robin", page = 0, per_page = 5)
#> $results
#> # A tibble: 5 x 14
#>   recordType elementGlobalId uniqueId nsxUrl elcode scientificName
#>   <chr>                <int> <chr>    <chr>  <chr>  <chr>         
#> 1 SPECIES             100637 ELEMENT… /Taxo… ABPBJ… Copsychus sau…
#> 2 SPECIES             102323 ELEMENT… /Taxo… ABPBJ… Turdus grayi  
#> 3 SPECIES             102179 ELEMENT… /Taxo… ABPBJ… Turdus migrat…
#> 4 SPECIES             105536 ELEMENT… /Taxo… ABPBJ… Turdus migrat…
#> 5 SPECIES             105850 ELEMENT… /Taxo… ABPBJ… Turdus rufopa…
#> # … with 22 more variables: formattedScientificName <chr>,
#> #   primaryCommonName <chr>, primaryCommonNameLanguage <chr>,
#> #   roundedGRank <chr>, nations <list>, lastModified <chr>,
#> #   speciesGlobal$usesaCode <lgl>, $cosewicCode <lgl>, $saraCode <lgl>,
#> #   $synonyms <list>, $otherCommonNames <list>, $kingdom <chr>, $phylum <chr>,
#> #   $taxclass <chr>, $taxorder <chr>, $family <chr>, $genus <chr>,
#> #   $taxonomicComments <chr>, $informalTaxonomy <chr>, $infraspecies <lgl>,
#> #   $completeDistribution <lgl>, gRank <chr>
#> 
#> $resultsSummary
#>                            name value
#> 1                          page     0
#> 2                recordsPerPage     5
#> 3                    totalPages    26
#> 4                  totalResults   126
#> 5                 species_total   103
#> 6                         total    23
#> 7                       classes     0
#> 8                    subclasses     0
#> 9                    formations     0
#> 10                    divisions     0
#> 11                  macrogroups     1
#> 12                       groups     1
#> 13                    alliances     3
#> 14                 associations    18
#> 15 terrestrialEcologicalSystems     0
```

Species search


```r
ns_search_spp(species_taxonomy = list(scientificTaxonomy = "Animalia", level = "kingdom"))
#> $results
#> # A tibble: 20 x 14
#>    recordType elementGlobalId uniqueId nsxUrl elcode scientificName
#>    <chr>                <int> <chr>    <chr>  <chr>  <chr>         
#>  1 SPECIES             828458 ELEMENT… /Taxo… AAABC… Acris blancha…
#>  2 SPECIES             828419 ELEMENT… /Taxo… AAABC… Acris crepita…
#>  3 SPECIES             103292 ELEMENT… /Taxo… AAABC… Acris gryllus 
#>  4 SPECIES             104324 ELEMENT… /Taxo… AAAAA… Ambystoma ann…
#>  5 SPECIES             100100 ELEMENT… /Taxo… AAAAA… Ambystoma bar…
#>  6 SPECIES             802300 ELEMENT… /Taxo… AAAAA… Ambystoma bis…
#>  7 SPECIES             104488 ELEMENT… /Taxo… AAAAA… Ambystoma cal…
#>  8 SPECIES             802301 ELEMENT… /Taxo… AAAAA… Ambystoma cin…
#>  9 SPECIES             103251 ELEMENT… /Taxo… AAAAA… Ambystoma gra…
#> 10 SPECIES             100401 ELEMENT… /Taxo… AAAAA… Ambystoma jef…
#> 11 SPECIES             102149 ELEMENT… /Taxo… AAAAA… Ambystoma lat…
#> 12 SPECIES            1143335 ELEMENT… /Taxo… AAAAA… Ambystoma lat…
#> 13 SPECIES            1143337 ELEMENT… /Taxo… AAAAA… Ambystoma lat…
#> 14 SPECIES             101261 ELEMENT… /Taxo… AAAAA… Ambystoma mab…
#> 15 SPECIES             106403 ELEMENT… /Taxo… AAAAA… Ambystoma mac…
#> 16 SPECIES             101632 ELEMENT… /Taxo… AAAAA… Ambystoma mac…
#> 17 SPECIES             103392 ELEMENT… /Taxo… AAAAA… Ambystoma mac…
#> 18 SPECIES             104239 ELEMENT… /Taxo… AAAAA… Ambystoma mac…
#> 19 SPECIES             100115 ELEMENT… /Taxo… AAAAA… Ambystoma mac…
#> 20 SPECIES             103039 ELEMENT… /Taxo… AAAAA… Ambystoma mac…
#> # … with 22 more variables: formattedScientificName <chr>,
#> #   primaryCommonName <chr>, primaryCommonNameLanguage <chr>,
#> #   roundedGRank <chr>, nations <list>, lastModified <chr>,
#> #   speciesGlobal$usesaCode <chr>, $cosewicCode <chr>, $saraCode <chr>,
#> #   $synonyms <list>, $otherCommonNames <list>, $kingdom <chr>, $phylum <chr>,
#> #   $taxclass <chr>, $taxorder <chr>, $family <chr>, $genus <chr>,
#> #   $taxonomicComments <chr>, $informalTaxonomy <chr>, $infraspecies <lgl>,
#> #   $completeDistribution <lgl>, gRank <chr>
#> 
#> $resultsSummary
#>             name value
#> 1           page     0
#> 2 recordsPerPage    20
#> 3     totalPages  2586
#> 4   totalResults 51705
#> 5  species_total 51705
```

Ecosystem search


```r
ns_search_eco(ecosystem_taxonomy = "M067")
#> $results
#> # A tibble: 20 x 14
#>    recordType elementGlobalId uniqueId nsxUrl elcode scientificName
#>    <chr>                <int> <chr>    <chr>  <chr>  <chr>         
#>  1 ECOSYSTEM           899525 ELEMENT… /Taxo… A3401  Eleocharis el…
#>  2 ECOSYSTEM           899121 ELEMENT… /Taxo… A1372  Fimbristylis …
#>  3 ECOSYSTEM           899126 ELEMENT… /Taxo… A1389  Spartina bake…
#>  4 ECOSYSTEM           899744 ELEMENT… /Taxo… A3692  Spartina pate…
#>  5 ECOSYSTEM           899523 ELEMENT… /Taxo… A3399  Typha dominge…
#>  6 ECOSYSTEM           899526 ELEMENT… /Taxo… A3402  Andropogon ca…
#>  7 ECOSYSTEM           899517 ELEMENT… /Taxo… A3393  Aristida palu…
#>  8 ECOSYSTEM           899132 ELEMENT… /Taxo… A1429  Eleocharis sp…
#>  9 ECOSYSTEM           899512 ELEMENT… /Taxo… A3388  Hypericum cha…
#> 10 ECOSYSTEM           899133 ELEMENT… /Taxo… A1430  Juncus milita…
#> 11 ECOSYSTEM           899122 ELEMENT… /Taxo… A1379  Panicum hemit…
#> 12 ECOSYSTEM           899124 ELEMENT… /Taxo… A1383  Rhynchospora …
#> 13 ECOSYSTEM           899518 ELEMENT… /Taxo… A3394  Rhynchospora …
#> 14 ECOSYSTEM          1146914 ELEMENT… /Taxo… A4404  Rhynchospora …
#> 15 ECOSYSTEM           899521 ELEMENT… /Taxo… A3397  Rhynchospora …
#> 16 ECOSYSTEM           899125 ELEMENT… /Taxo… A1384  Rhynchospora …
#> 17 ECOSYSTEM           899530 ELEMENT… /Taxo… A3406  Cladium maris…
#> 18 ECOSYSTEM           899120 ELEMENT… /Taxo… A1369  Cladium maris…
#> 19 ECOSYSTEM           899720 ELEMENT… /Taxo… A3668  Hottonia infl…
#> 20 ECOSYSTEM           900115 ELEMENT… /Taxo… A4065  Orontium aqua…
#> # … with 18 more variables: formattedScientificName <chr>,
#> #   primaryCommonName <chr>, primaryCommonNameLanguage <chr>,
#> #   roundedGRank <chr>, nations <list>, lastModified <chr>,
#> #   ecosystemGlobal$translatedScientificName <chr>, $taxclassCode <chr>,
#> #   $taxsubclassCode <chr>, $formationCode <chr>, $divisionCode <chr>,
#> #   $macrogroupKey <chr>, $taxgroupKey <chr>, $allianceKey <lgl>,
#> #   $ecosystemType <chr>, $classificationCode <chr>, $parentName <chr>,
#> #   gRank <chr>
#> 
#> $resultsSummary
#>                            name value
#> 1                          page     0
#> 2                recordsPerPage    20
#> 3                    totalPages    11
#> 4                  totalResults   210
#> 5                         total   210
#> 6                       classes     0
#> 7                    subclasses     0
#> 8                    formations     0
#> 9                     divisions     0
#> 10                  macrogroups     1
#> 11                       groups     6
#> 12                    alliances    33
#> 13                 associations   154
#> 14 terrestrialEcologicalSystems    16
```

## Get taxon by id

By UID


```r
w <- ns_id("ELEMENT_GLOBAL.2.154701")
str(w, max.level = 1)
#> List of 40
#>  $ elementGlobalId                        : int 154701
#>  $ circumscripConfidence                  : NULL
#>  $ classificationLevel                    :List of 4
#>  $ classificationStatus                   :List of 4
#>  $ iucn                                   :List of 5
#>  $ nameCategory                           :List of 6
#>  $ rankMethodUsed                         :List of 4
#>  $ formattedScientificName                : chr "<i>Hydrastis canadensis</i>"
#>  $ scientificName                         : chr "Hydrastis canadensis"
#>  $ scientificNameAuthor                   : chr "L."
#>  $ primaryCommonName                      : chr "Goldenseal"
#>  $ relatedItisNames                       : chr "<i>Hydrastis canadensis</i> L. (TSN 18781)"
#>  $ uniqueId                               : chr "ELEMENT_GLOBAL.2.154701"
#>  $ elcode                                 : chr "PDRAN0F010"
#>  $ conceptRefFullCitation                 : chr "Kartesz, J.T. 1994. A synonymized checklist of the vascular flora of the United States, Canada, and Greenland. "| __truncated__
#>  $ conceptName                            : chr "<i>Hydrastis canadensis</i>"
#>  $ taxonomicComments                      : chr "<i>Hydrastis canadensis</i> occurs in eastern North America and is a monotypic genus. In the most current taxon"| __truncated__
#>  $ roundedGRank                           : chr "G3"
#>  $ conservationStatusFactorsEditionDate   : chr "2013-04-29"
#>  $ conservationStatusFactorsEditionAuthors: chr "Oliver, L."
#>  $ primaryCommonNameLanguage              : chr "EN"
#>  $ recordType                             : chr "SPECIES"
#>  $ elementNationals                       :'data.frame':	2 obs. of  7 variables:
#>  $ lastModified                           : chr "2020-05-14T04:31:58.480462Z"
#>  $ lastPublished                          : chr "2020-05-14T02:14:14.569584Z"
#>  $ nsxUrl                                 : chr "/Taxon/ELEMENT_GLOBAL.2.154701/Hydrastis_canadensis"
#>  $ grank                                  : chr "G3G4"
#>  $ grankReviewDate                        : chr "2012-11-30"
#>  $ grankChangeDate                        : chr "2012-11-30"
#>  $ grankReasons                           : chr "Goldenseal, <i>Hydrastis canadensis, </i>an herbaceous understory species of the eastern deciduous forest, with"| __truncated__
#>  $ rankInfo                               :List of 27
#>  $ animalCharacteristics                  : NULL
#>  $ occurrenceDelineations                 :'data.frame':	1 obs. of  17 variables:
#>  $ plantCharacteristics                   :List of 7
#>  $ elementManagement                      :List of 14
#>  $ occurrenceViabilities                  :'data.frame':	1 obs. of  12 variables:
#>  $ references                             :'data.frame':	97 obs. of  6 variables:
#>  $ otherCommonNames                       :'data.frame':	6 obs. of  3 variables:
#>  $ speciesGlobal                          :List of 29
#>  $ speciesCharacteristics                 :List of 13
```

By alternate id


```r
x <- ns_altid(uid = "ELEMENT_GLOBAL.2.154701")
str(x, max.level = 1)
#> List of 40
#>  $ elementGlobalId                        : int 154701
#>  $ circumscripConfidence                  : NULL
#>  $ classificationLevel                    :List of 4
#>  $ classificationStatus                   :List of 4
#>  $ iucn                                   :List of 5
#>  $ nameCategory                           :List of 6
#>  $ rankMethodUsed                         :List of 4
#>  $ formattedScientificName                : chr "<i>Hydrastis canadensis</i>"
#>  $ scientificName                         : chr "Hydrastis canadensis"
#>  $ scientificNameAuthor                   : chr "L."
#>  $ primaryCommonName                      : chr "Goldenseal"
#>  $ relatedItisNames                       : chr "<i>Hydrastis canadensis</i> L. (TSN 18781)"
#>  $ uniqueId                               : chr "ELEMENT_GLOBAL.2.154701"
#>  $ elcode                                 : chr "PDRAN0F010"
#>  $ conceptRefFullCitation                 : chr "Kartesz, J.T. 1994. A synonymized checklist of the vascular flora of the United States, Canada, and Greenland. "| __truncated__
#>  $ conceptName                            : chr "<i>Hydrastis canadensis</i>"
#>  $ taxonomicComments                      : chr "<i>Hydrastis canadensis</i> occurs in eastern North America and is a monotypic genus. In the most current taxon"| __truncated__
#>  $ roundedGRank                           : chr "G3"
#>  $ conservationStatusFactorsEditionDate   : chr "2013-04-29"
#>  $ conservationStatusFactorsEditionAuthors: chr "Oliver, L."
#>  $ primaryCommonNameLanguage              : chr "EN"
#>  $ recordType                             : chr "SPECIES"
#>  $ elementNationals                       :'data.frame':	2 obs. of  7 variables:
#>  $ lastModified                           : chr "2020-05-14T04:31:58.480462Z"
#>  $ lastPublished                          : chr "2020-05-14T02:14:14.569584Z"
#>  $ nsxUrl                                 : chr "/Taxon/ELEMENT_GLOBAL.2.154701/Hydrastis_canadensis"
#>  $ grank                                  : chr "G3G4"
#>  $ grankReviewDate                        : chr "2012-11-30"
#>  $ grankChangeDate                        : chr "2012-11-30"
#>  $ grankReasons                           : chr "Goldenseal, <i>Hydrastis canadensis, </i>an herbaceous understory species of the eastern deciduous forest, with"| __truncated__
#>  $ rankInfo                               :List of 27
#>  $ animalCharacteristics                  : NULL
#>  $ occurrenceDelineations                 :'data.frame':	1 obs. of  17 variables:
#>  $ plantCharacteristics                   :List of 7
#>  $ elementManagement                      :List of 14
#>  $ occurrenceViabilities                  :'data.frame':	1 obs. of  12 variables:
#>  $ references                             :'data.frame':	97 obs. of  6 variables:
#>  $ otherCommonNames                       :'data.frame':	6 obs. of  3 variables:
#>  $ speciesGlobal                          :List of 29
#>  $ speciesCharacteristics                 :List of 13
```

## Get a summary of the upper level hierarchy for an Ecosystem record


```r
ns_ecohier("ELEMENT_GLOBAL.2.683060")
#>                  uniqueId
#> 1 ELEMENT_GLOBAL.2.860217
#> 2 ELEMENT_GLOBAL.2.860227
#> 3 ELEMENT_GLOBAL.2.860241
#> 4 ELEMENT_GLOBAL.2.860284
#> 5 ELEMENT_GLOBAL.2.838501
#> 6 ELEMENT_GLOBAL.2.833279
#> 7 ELEMENT_GLOBAL.2.899395
#>                                                            name
#> 1                                             Forest & Woodland
#> 2                          Temperate & Boreal Forest & Woodland
#> 3                              Cool Temperate Forest & Woodland
#> 4                      Eastern North American Forest & Woodland
#> 5         Southern & South-Central Oak - Pine Forest & Woodland
#> 6 South-Central Interior Shortleaf Pine - Oak Forest & Woodland
#> 7                  Ozark-Ouachita Shortleaf Pine - Oak Woodland
#>                                                                                                                 nsxUrl
#> 1                                                     /Taxon/ELEMENT_GLOBAL.2.860217/Mesomorphic_Tree_Vegetation_Class
#> 2                                             /Taxon/ELEMENT_GLOBAL.2.860227/Temperate_Boreal_Forest_Woodland_Subclass
#> 3                                              /Taxon/ELEMENT_GLOBAL.2.860241/Cool_Temperate_Forest_Woodland_Formation
#> 4           /Taxon/ELEMENT_GLOBAL.2.860284/Acer_saccharum_-_Fagus_grandifolia_-_Quercus_rubra_Forest_Woodland_Division
#> 5            /Taxon/ELEMENT_GLOBAL.2.838501/Quercus_alba_-_Quercus_falcata_-_Pinus_echinata_Forest_Woodland_Macrogroup
#> 6             /Taxon/ELEMENT_GLOBAL.2.833279/Pinus_echinata_-_Quercus_falcata_-_Quercus_stellata_Forest_Woodland_Group
#> 7 /Taxon/ELEMENT_GLOBAL.2.899395/Pinus_echinata_-_Quercus_stellata_-_Quercus_velutina_Ozark-Ouachita_Woodland_Alliance
#>   ecosystemType classificationCode
#> 1         CLASS                  1
#> 2      SUBCLASS                1.B
#> 3     FORMATION              1.B.2
#> 4      DIVISION           1.B.2.Na
#> 5    MACROGROUP               M016
#> 6         GROUP               G012
#> 7      ALLIANCE              A3271
```

## Search exports

`ns_export()` uses the same search interface as the `ns_search*` functions, but instead of downloading data immediately, `ns_export()` creates a "download job", which eventually provides a compressed JSON file that you can download.


```r
x <- ns_export(text = "robin")
x
#> [1] "7a107bea-b98d-4b5a-87b3-456ea2194f07"
```

You can pass the output of `ns_export()` to `ns_export_status()` to get the status of the job


```r
res <- ns_export_status(x)
#> $state
#> [1] "Finished"
#> 
#> $data
#> $data$success
#> [1] TRUE
#> $percentComplete
#> [1] 100
#> $successful
#> [1] TRUE
#> $error
#> ...
```

When state equals "Finished", you can read the data into R, e.g, with `jsonlite`:


```r
res$data$url
#> [1] "https://explorer-downloads.natureserve.org/shortTerm/explorer/taxaSearchExports/2020-05-15/7a107bea-b98d-4b5a-87b3-456ea2194f07.json"
tibble::as_tibble(jsonlite::fromJSON(res$data$url))
#> # A tibble: 126 x 14
#>    elementGlobalId uniqueId nsxUrl elcode scientificName formattedScient… primaryCommonNa… primaryCommonNa… roundedGRank
#>              <int> <chr>    <chr>  <chr>  <chr>          <chr>            <chr>            <chr>            <chr>
#>  1          100637 ELEMENT… /Taxo… ABPBJ… Copsychus sau… <i>Copsychus sa… Oriental Magpie… EN               G5
#>  2          102323 ELEMENT… /Taxo… ABPBJ… Turdus grayi   <i>Turdus grayi… Clay-colored Th… EN               G5
#>  3          102179 ELEMENT… /Taxo… ABPBJ… Turdus migrat… <i>Turdus migra… American Robin   EN               G5
#>  4          105536 ELEMENT… /Taxo… ABPBJ… Turdus migrat… <i>Turdus migra… Western America… EN               TU
#>  5          105850 ELEMENT… /Taxo… ABPBJ… Turdus rufopa… <i>Turdus rufop… Rufous-backed R… EN               G5
#>  6          100589 ELEMENT… /Taxo… AFC4B… Peristedion g… <i>Peristedion … Slender Searobin EN               GNR
#>  7          105826 ELEMENT… /Taxo… AFC4B… Prionotus ala… <i>Prionotus al… Spiny Searobin   EN               GNR
#>  8          101394 ELEMENT… /Taxo… AFC4B… Prionotus car… <i>Prionotus ca… Northern Searob… EN               G5
#>  9          100276 ELEMENT… /Taxo… AFC4B… Prionotus evo… <i>Prionotus ev… Striped Searobin EN               G5
#> 10          103595 ELEMENT… /Taxo… AFC4B… Prionotus lon… <i>Prionotus lo… Bigeye Searobin  EN               G5
#> # … with 116 more rows, and 29 more variables: nations <list>, lastModified <chr>, speciesGlobal$usesaCode <chr>,
#> #   $cosewicCode <chr>, $saraCode <chr>, $synonyms <list>, $otherCommonNames <list>, $kingdom <chr>, $phylum <chr>,
#> #   $taxclass <chr>, $taxorder <chr>, $family <chr>, $genus <chr>, $taxonomicComments <chr>, $informalTaxonomy <chr>,
#> #   $infraspecies <lgl>, $completeDistribution <lgl>, gRank <chr>, ecosystemGlobal$translatedScientificName <chr>,
#> #   $taxclassCode <chr>, $taxsubclassCode <chr>, $formationCode <chr>, $divisionCode <chr>, $macrogroupKey <chr>,
#> #   $taxgroupKey <chr>, $allianceKey <chr>, $ecosystemType <chr>, $classificationCode <chr>, $parentName <chr>
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ns_search_comb.R
\name{ns_search_comb}
\alias{ns_search_comb}
\title{Combined search}
\usage{
ns_search_comb(
  text = NULL,
  text_adv = NULL,
  status = NULL,
  location = NULL,
  record_type = NULL,
  record_subtype = NULL,
  modified_since = NULL,
  page = NULL,
  per_page = NULL,
  ...
)
}
\arguments{
\item{text}{(character) basic text search, equiavalent to \code{text_adv}
with \code{matchAgainst="allNames"} and \code{operator="similarTo"}}

\item{text_adv}{(list) advanced search, must specify the following three
elements: \code{searchToken}, \code{matchAgainst}, and \code{operator}. see
https://explorer.natureserve.org/api-docs/#_advanced_text_search_parameter}

\item{status}{(character) conservation status, one of G1, G2, G3, G4,
G5, GH, GX, GNR, GNA, GU. case insensitive}

\item{location}{(list) location, country and sub-country. specify either
\code{nation} OR \code{nation} and \code{subnation}. each expects a two-letter ISO code}

\item{record_type}{(character) limit results by record type, one of
"species" or "ecosystem"}

\item{record_subtype}{(character) limit results by record sub-type, one of:
"class", "subclass", "formation", "division", "macrogroup", "group",
"alliance", "association", "terrestrial_ecological_system"}

\item{modified_since}{(character) search for records modified since a
given time. value must be a date and time with a UTC offset in ISO 8601
format. optional}

\item{page}{(integer) Zero-indexed page number; default: 0. optional}

\item{per_page}{(integer) Records per page; default: 20. optional}

\item{...}{Curl options passed on to \code{\link[crul]{verb-GET}}}
}
\description{
Combined search
}
\examples{
\dontrun{
ns_search_comb(text = "robin")
ns_search_comb(text_adv = list(searchToken = "western",
  matchAgainst="allScientificNames", operator="startsWith"))
ns_search_comb(status = "G1")
ns_search_comb(location = list(nation = "US"))
ns_search_comb(location = list(nation = "US", subnation = "VA"))
ns_search_comb(record_type = "species")
ns_search_comb(record_subtype = "macrogroup")
ns_search_comb(modified_since = "2020-04-30T00:00:00+0000")
ns_search_comb(page = 0, per_page = 2)
}
}
\references{
https://explorer.natureserve.org/api-docs/
}
\seealso{
Other search: 
\code{\link{ns_search_eco}()},
\code{\link{ns_search_spp}()}
}
\concept{search}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/natserv-package.R
\docType{data}
\name{nat_states}
\alias{nat_states}
\title{A data.frame with 49 rows and 2 columns}
\description{
\itemize{
\item state (character) state 2 letter abbreviation
\item state_name (character) state full name
}
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ns_search.R
\name{ns_search}
\alias{ns_search}
\title{NatureServe taxonomic name search}
\usage{
ns_search(x, key = NULL, ...)
}
\arguments{
\item{x}{(character) A name to search for. An asterisk (*) wildcarded species
name, e.g., 'Aquila chry*'. Name matching is case-insensitive and all of
the primary and synonymous scientific names, along with all common names,
are matched. Required.}

\item{key}{(character) API key. Required. See \strong{Authentication} below
for more.}

\item{...}{Curl options passed on to \code{\link[crul]{HttpClient}}}
}
\value{
A tibble (data.frame), with columns:
\itemize{
 \item jurisdictionScientificName - Scientfic name
 \item commonName - Common name
 \item globalSpeciesUid - UID - the taxonomic identifier NatureServe uses
 \item natureServeExplorerURI - URL to get to info online for the taxon
 \item taxonomicComments - comments about the taxon, if any
}
}
\description{
NatureServe taxonomic name search
}
\section{Authentication}{

Get an API key from NatureServe at
\url{https://services.natureserve.org/developer/index.jsp}.
You can pass your token in as an argument or store it one of two places:

\itemize{
  \item your .Rprofile file with an entry like
  \code{options(NatureServeKey = "your-natureserve-key")}
  \item your .Renviron file with an entry like
  \code{NATURE_SERVE_KEY=your-natureserve-key}
}

See \code{\link{Startup}} for information on how to create/find your
.Rrofile and .Renviron files
}

\examples{
\dontrun{
ns_search(x = "Ruby*")
ns_search(x = "Helianthus annuus")
ns_search(x = "Ursus americanus")
}
}
\references{
\url{https://services.natureserve.org/index.jsp}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ns_export.R
\name{ns_export}
\alias{ns_export}
\alias{ns_export_status}
\title{Search exports}
\usage{
ns_export(
  text = NULL,
  text_adv = NULL,
  status = NULL,
  location = NULL,
  record_type = NULL,
  record_subtype = NULL,
  modified_since = NULL,
  format = "json",
  lang = "en",
  ...
)

ns_export_status(id, ...)
}
\arguments{
\item{text}{(character) basic text search, equiavalent to \code{text_adv}
with \code{matchAgainst="allNames"} and \code{operator="similarTo"}}

\item{text_adv}{(list) advanced search, must specify the following three
elements: \code{searchToken}, \code{matchAgainst}, and \code{operator}. see
https://explorer.natureserve.org/api-docs/#_advanced_text_search_parameter}

\item{status}{(character) conservation status, one of G1, G2, G3, G4,
G5, GH, GX, GNR, GNA, GU. case insensitive}

\item{location}{(list) location, country and sub-country. specify either
\code{nation} OR \code{nation} and \code{subnation}. each expects a two-letter ISO code}

\item{record_type}{(character) limit results by record type, one of
"species" or "ecosystem"}

\item{record_subtype}{(character) limit results by record sub-type, one of:
"class", "subclass", "formation", "division", "macrogroup", "group",
"alliance", "association", "terrestrial_ecological_system"}

\item{modified_since}{(character) search for records modified since a
given time. value must be a date and time with a UTC offset in ISO 8601
format. optional}

\item{format}{(character) output format, one of "json" or "xlsx"}

\item{lang}{(character) language, one of "en", "es", or "fr"}

\item{...}{Curl options passed on to \code{\link[crul]{verb-GET}}}

\item{id}{(character) a job id, from output of \code{ns_export()}}
}
\value{
\code{ns_export()} returns a single character string (a job id)
\code{ns_export_status()} returns a list of metadata concerning the
status of the export
}
\description{
Search exports
}
\examples{
\dontrun{
x <- ns_export(text = "robin")
res <- ns_export_status(x)
str(res)
res$state
res$data$errorMessage
res$data$url

w <- ns_export(text_adv = list(searchToken = "western",
  matchAgainst="allScientificNames", operator="startsWith"))
m <- ns_export_status(w)
head(jsonlite::fromJSON(m$data$url))
}
}
\references{
https://explorer.natureserve.org/api-docs/
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ns_map.R
\name{ns_map}
\alias{ns_map}
\alias{ns_map_county}
\alias{ns_map_cons}
\alias{ns_map_stpr}
\title{Map NatureServe data}
\usage{
ns_map_county(x, ...)

ns_map_cons(x, ...)

ns_map_stpr(x, ...)
}
\arguments{
\item{x}{the result of a call to \code{\link{ns_data}}}

\item{...}{ignored}
}
\description{
Map NatureServe data
}
\examples{
\dontrun{
# Aquila chrysaetos
x <- ns_data(uid = 'ELEMENT_GLOBAL.2.100925')
ns_map_county(x)
ns_map_cons(x)
ns_map_stpr(x)

# Ursus americanus
x <- ns_data('ELEMENT_GLOBAL.2.100661')
ns_map_county(x)
ns_map_cons(x)
ns_map_stpr(x)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ns_altid.R
\name{ns_altid}
\alias{ns_altid}
\title{Get taxon by uid, id, or elCode}
\usage{
ns_altid(uid = NULL, id = NULL, el_code = NULL, ...)
}
\arguments{
\item{uid}{(character) A NatureServe taxon id (The taxon’s Element Global
UID)}

\item{id}{The primary key value (ELEMENT_GLOBAL_ID) of the record within
Central Biotics}

\item{el_code}{The Biotics Element Code (ELCODE_BCD) of the record}

\item{...}{Curl options passed on to \code{\link[crul]{verb-GET}}}
}
\value{
A list with lots of elements
}
\description{
Get taxon by uid, id, or elCode
}
\details{
see https://explorer.natureserve.org/api-docs/#_taxon_data_model
for details on the response data
}
\examples{
\dontrun{
ns_altid(uid = "ELEMENT_GLOBAL.2.154701")
ns_altid(id = "154701")
ns_altid(el_code = "PDRAN0F010")
}
}
\references{
https://explorer.natureserve.org/api-docs/
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ns_data.R
\name{ns_data}
\alias{ns_data}
\title{NatureServe data}
\usage{
ns_data(uid, key = NULL, ...)
}
\arguments{
\item{uid}{(character) a species UID, e.g., ELEMENT_GLOBAL.2.100925}

\item{key}{(character) API key. Required. See \strong{Authentication} below
for more.}

\item{...}{Curl options passed on to \code{\link[crul]{HttpClient}}}
}
\value{
a named list, with possible slots:
\itemize{
 \item natureserve_uri
 \item classification
 \item economicAttributes
 \item license
 \item references
 \item conservationStatus
 \item managementSummary
 \item distribution
 \item ecologyAndLifeHistory
}
}
\description{
NatureServe data
}
\section{Authentication}{

Get an API key from NatureServe at
\url{https://services.natureserve.org/developer/index.jsp}.
You can pass your token in as an argument or store it one of two places:

\itemize{
  \item your .Rprofile file with an entry like
  \code{options(NatureServeKey = "your-natureserve-key")}
  \item your .Renviron file with an entry like
  \code{NATURE_SERVE_KEY=your-natureserve-key}
}

See \code{\link{Startup}} for information on how to create/find your
.Rrofile and .Renviron files
}

\section{National (N) and Subnational (S) Conservation Status Ranks}{

\itemize{
 \item NX,SX - Presumed Extirpated - Species or community is believed to be
 extirpated from the nation or state/province. Not located despite intensive
 searches of historical sites and other appropriate habitat, and virtually
 no likelihood that it will be rediscovered.
 \item NH,SH - Possibly Extirpated (Historical) - Species or community
 occurred historically in the nation or state/province, and there is some
 possibility that it may be rediscovered. Its presence may not have been
 verified in the past 20-40 years. A species or community could become NH
 or SH without such a 20-40 year delay if the only known occurrences in a
 nation or state/province were destroyed or if it had been extensively and
 unsuccessfully looked for. The NH or SH rank is reserved for species or
 communities for which some effort has been made to relocate occurrences,
 rather than simply using this status for all elements not known from
 verified extant occurrences.
 \item N1,S1 - Critically Imperiled - Critically imperiled in the nation or
 state/province because of extreme rarity (often 5 or fewer occurrences)
 or because of some factor(s) such as very steep declines making it
 especially vulnerable to extirpation from the state/province.
 \item N2,S2 - Imperiled - Imperiled in the nation or state/province because
 of rarity due to very restricted range, very few populations (often 20 or
 fewer), steep declines, or other factors making it very vulnerable to
 extirpation from the nation or state/province.
 \item N3,S3 - Vulnerable - Vulnerable in the nation or state/province due
 to a restricted range, relatively few populations (often 80 or fewer),
 recent and widespread declines, or other factors making it vulnerable
 to extirpation.
 \item N4,S4 - Apparently Secure - Uncommon but not rare; some cause for
 long-term concern due to declines or other factors.
 \item N5,S5 - Secure - Common, widespread, and abundant in the nation or
 state/province.
 \item NNR,SNR - Unranked - Nation or state/province conservation status
 not yet assessed.
 \item NU,SU - Unrankable - Currently unrankable due to lack of information
 or due to substantially conflicting information about status or trends.
 \item NNA,SNA - Not Applicable - A conservation status rank is not
 applicable because the species is not a suitable target for conservation
 activities.
}
}

\section{Breeding Status Qualifiers}{

\itemize{
 \item B - Breeding - Conservation status refers to the breeding population of
 the species in the nation or state/province.
 \item N - Nonbreeding - Conservation status refers to the non-breeding
 population of the species in the nation or state/province.
 \item M - Migrant - Migrant species occurring regularly on migration at
 particular staging areas or concentration spots where the species might
 warrant conservation attention. Conservation status refers to the
 aggregating transient population of the species in the nation or
 state/province.
}
}

\examples{
\dontrun{
## single id
(res <- ns_data(uid = 'ELEMENT_GLOBAL.2.100925'))
## many ids at once
res <- ns_data(uid = c('ELEMENT_GLOBAL.2.100925', 'ELEMENT_GLOBAL.2.104470'))
res$ELEMENT_GLOBAL.2.100925
res$ELEMENT_GLOBAL.2.104470

ns_data(uid = 'ELEMENT_GLOBAL.2.101998')
}
}
\references{
\url{https://services.natureserve.org/index.jsp}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ns_images.R
\name{ns_images}
\alias{ns_images}
\title{NatureServe image metadata}
\usage{
ns_images(uid = NULL, scientificName = NULL, commonName = NULL,
  includeSynonyms = NULL, resolution = NULL, ITISNames = NULL,
  key = NULL, ...)
}
\arguments{
\item{uid}{(character) a species UID, e.g., ELEMENT_GLOBAL.2.100925}

\item{scientificName}{(character) An asterisk (*) wildcarded species
scientific name, e.g., 'Aquila chry*'. Name matching is case-insensitive.}

\item{commonName}{(character) An asterisk (*) wildcarded species common
name, e.g., 'g*EAGLE'. Name matching is case-insensitive.}

\item{includeSynonyms}{(character)    An optional parameter, relevant to
scientific or common name queries, that indicates whether to include
synonymous names in the query, as follows:
\itemize{
 \item Y (or y) - search the Primary and all synonymous Scientific and
 Common Names
 \item Any other value, or omitted - search only the Primary Scientific
 and Common Name fields
}}

\item{resolution}{(character)    An optional parameter that restricts output
to images at a certain resolution. The value can be of one of the following.
\itemize{
 \item lowest - return only the lowest resolution version of images
 \item highest - return only the highest resolution version of images
 \item thumbnail - return only the version NatureServe deems the
 'thumbnail image'
 \item web - return only the version NatureServe deems the 'web image'
}
The omission of this parameter leads to the return of metadata for images at
all available resolutions.}

\item{ITISNames}{(character)    An optional parameter, relevant to scientific
or common name queries, that will indicate whether to restrict queries to
ITIS names, as follows.
\itemize{
 \item Y (or y) - query using ITIS names
 \item Any other value, or omitted - query using NatureServe names
}
NOTE: This parameter is a placeholder only and will not affect processing.
At present, searching by ITIS names is not possible.}

\item{key}{(character) API key. Required. See \strong{Authentication} below
for more.}

\item{...}{Curl options passed on to \code{\link[crul]{HttpClient}}}
}
\value{
a list with terms and images
}
\description{
NatureServe image metadata
}
\details{
Note that the NatureServer servers apparently want Windows
HTML encoding (Windows-1252), instead of UTF-8, so some accents
and such may not work
}
\section{Authentication}{

Get an API key from NatureServe at
\url{https://services.natureserve.org/developer/index.jsp}.
You can pass your token in as an argument or store it one of two places:

\itemize{
  \item your .Rprofile file with an entry like
  \code{options(NatureServeKey = "your-natureserve-key")}
  \item your .Renviron file with an entry like
  \code{NATURE_SERVE_KEY=your-natureserve-key}
}

See \code{\link{Startup}} for information on how to create/find your
.Rrofile and .Renviron files
}

\examples{
\dontrun{
# search by uid
ns_images(uid = 'ELEMENT_GLOBAL.2.100925')

# search by common name and resolutio thumbnail
(res <- ns_images(commonName = "*eagle", resolution = 'thumbnail'))

# search "Ruby*", all common names [in any language], and highest
# resolution only:
(res <- ns_images(commonName = "Ruby*", includeSynonyms = 'y',
  resolution = 'highest'))
}
}
\references{
\url{https://services.natureserve.org/index.jsp}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ns_ecohier.R
\name{ns_ecohier}
\alias{ns_ecohier}
\title{Get a summary of the upper level hierarchy for an Ecosystem record}
\usage{
ns_ecohier(uid, ...)
}
\arguments{
\item{uid}{(character) A NatureServe taxon id (The taxon’s Element Global
UID)}

\item{...}{Curl options passed on to \code{\link[crul]{verb-GET}}}
}
\value{
A list with lots of elements
}
\description{
Get a summary of the upper level hierarchy for an Ecosystem record
}
\details{
see https://explorer.natureserve.org/api-docs/#_taxon_data_model
for details on the response data
}
\examples{
\dontrun{
ns_ecohier("ELEMENT_GLOBAL.2.683060")
}
}
\references{
https://explorer.natureserve.org/api-docs/
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ns_id.R
\name{ns_id}
\alias{ns_id}
\title{Get taxon by uid}
\usage{
ns_id(uid, ...)
}
\arguments{
\item{uid}{(character) A NatureServe taxon id (The taxon’s Element Global
UID). required.}

\item{...}{Curl options passed on to \code{\link[crul]{verb-GET}}}
}
\value{
A list with lots of elements
}
\description{
Get taxon by uid
}
\details{
see https://explorer.natureserve.org/api-docs/#_taxon_data_model
for details on the response data
}
\examples{
\dontrun{
ns_id("ELEMENT_GLOBAL.2.154701")
}
}
\references{
https://explorer.natureserve.org/api-docs/
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/natserv-package.R
\docType{package}
\name{natserv-package}
\alias{natserv-package}
\alias{natserv}
\title{natserv}
\description{
Interface to NatureServe https://www.natureserve.org/
}
\author{
Scott Chamberlain \email{myrmecocystus@gmail.com}
}
\keyword{package}
